# encoding: utf-8

# The MIT License
#
# Copyright (c) 2015 the bpython authors.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
"""simple evaluation of side-effect free code

In order to provide fancy completion, some code can be executed safely.

"""

from __future__ import absolute_import

import ast
import inspect
from six import string_types
from six.moves import builtins
import sys
import types

from . import line as line_properties
from ._py3compat import py3
from .inspection import is_new_style, AttrCleaner

_string_type_nodes = (ast.Str, ast.Bytes) if py3 else (ast.Str,)
_numeric_types = (int, float, complex) + (() if py3 else (long,))

# added in Python 3.4
if hasattr(ast, 'NameConstant'):
    _name_type_nodes = (ast.Name, ast.NameConstant)
else:
    _name_type_nodes = (ast.Name,)


class EvaluationError(Exception):
    """Raised if an exception occurred in safe_eval."""


def safe_eval(expr, namespace):
    """Not all that safe, just catches some errors"""
    try:
        return eval(expr, namespace)
    except (NameError, AttributeError, SyntaxError):
        # If debugging safe_eval, raise this!
        # raise
        raise EvaluationError


# This function is under the Python License, Version 2
# This license requires modifications to the code be reported.
# Based on ast.literal_eval in Python 2 and Python 3
# Modifications:
# * Python 2 and Python 3 versions of the function are combined
# * checks that objects used as operands of + and - are numbers
#   instead of checking they are constructed with number literals
# * new docstring describing different functionality
# * looks up names from namespace
# * indexing syntax is allowed
def simple_eval(node_or_string, namespace=None):
    """
    Safely evaluate an expression node or a string containing a Python
    expression without triggering any user code.

    The string or node provided may only consist of:
    * the following Python literal structures: strings, numbers, tuples,
        lists, and dicts
    * variable names causing lookups in the passed in namespace or builtins
    * getitem calls using the [] syntax on objects of the types above

    Like the Python 3 (and unlike the Python 2) literal_eval, unary and binary
    + and - operations are allowed on all builtin numeric types.

    The optional namespace dict-like ought not to cause side effects on lookup
    """
    if namespace is None:
        namespace = {}
    if isinstance(node_or_string, string_types):
        node_or_string = ast.parse(node_or_string, mode='eval')
    if isinstance(node_or_string, ast.Expression):
        node_or_string = node_or_string.body

    def _convert(node):
        if isinstance(node, _string_type_nodes):
            return node.s
        elif isinstance(node, ast.Num):
            return node.n
        elif isinstance(node, ast.Tuple):
            return tuple(map(_convert, node.elts))
        elif isinstance(node, ast.List):
            return list(map(_convert, node.elts))
        elif isinstance(node, ast.Dict):
            return dict((_convert(k), _convert(v)) for k, v
                        in zip(node.keys, node.values))

        # this is a deviation from literal_eval: we allow non-literals
        elif isinstance(node, _name_type_nodes):
            try:
                return namespace[node.id]
            except KeyError:
                try:
                    return getattr(builtins, node.id)
                except AttributeError:
                    raise EvaluationError("can't lookup %s" % node.id)

        # unary + and - are allowed on any type
        elif (isinstance(node, ast.UnaryOp) and
              isinstance(node.op, (ast.UAdd, ast.USub))):
            # ast.literal_eval does ast typechecks here, we use type checks
            operand = _convert(node.operand)
            if not type(operand) in _numeric_types:
                raise ValueError("unary + and - only allowed on builtin nums")
            if isinstance(node.op, ast.UAdd):
                return + operand
            else:
                return - operand
        elif (isinstance(node, ast.BinOp) and
              isinstance(node.op, (ast.Add, ast.Sub))):
            # ast.literal_eval does ast typechecks here, we use type checks
            left = _convert(node.left)
            right = _convert(node.right)
            if not (type(left) in _numeric_types and
                    type(right) in _numeric_types):
                raise ValueError("binary + and - only allowed on builtin nums")
            if isinstance(node.op, ast.Add):
                return left + right
            else:
                return left - right

        # this is a deviation from literal_eval: we allow indexing
        elif (isinstance(node, ast.Subscript) and
              isinstance(node.slice, ast.Index)):
            obj = _convert(node.value)
            index = _convert(node.slice.value)
            return safe_getitem(obj, index)

        # this is a deviation from literal_eval: we allow attribute access
        if isinstance(node, ast.Attribute):
            obj = _convert(node.value)
            attr = node.attr
            return safe_get_attribute(obj, attr)

        raise ValueError('malformed string')
    return _convert(node_or_string)


def safe_getitem(obj, index):
    if type(obj) in (list, tuple, dict, bytes) + string_types:
        try:
            return obj[index]
        except (KeyError, IndexError):
            raise EvaluationError("can't lookup key %r on %r" % (index, obj))
    raise ValueError('unsafe to lookup on object of type %s' % (type(obj), ))


def find_attribute_with_name(node, name):
    if isinstance(node, ast.Attribute) and node.attr == name:
        return node
    for item in ast.iter_child_nodes(node):
        r = find_attribute_with_name(item, name)
        if r:
            return r


def evaluate_current_expression(cursor_offset, line, namespace=None):
    """
    Return evaluated expression to the right of the dot of current attribute.

    Only evaluates builtin objects, and do any attribute lookup.
    """
    # Builds asts from with increasing numbers of characters back from cursor.
    # Find the biggest valid ast.
    # Once our attribute access is found, return its .value subtree

    if namespace is None:
        namespace = {}

    # in case attribute is blank, e.g. foo.| -> foo.xxx|
    temp_line = line[:cursor_offset] + 'xxx' + line[cursor_offset:]
    temp_cursor = cursor_offset + 3
    temp_attribute = line_properties.current_expression_attribute(
        temp_cursor, temp_line)
    if temp_attribute is None:
        raise EvaluationError("No current attribute")
    attr_before_cursor = temp_line[temp_attribute.start:temp_cursor]

    def parse_trees(cursor_offset, line):
        for i in range(cursor_offset - 1, -1, -1):
            try:
                tree = ast.parse(line[i:cursor_offset])
                yield tree
            except SyntaxError:
                continue

    largest_ast = None
    for tree in parse_trees(temp_cursor, temp_line):
        attribute_access = find_attribute_with_name(tree, attr_before_cursor)
        if attribute_access:
            largest_ast = attribute_access.value

    if largest_ast is None:
        raise EvaluationError(
            "Corresponding ASTs to right of cursor are invalid")
    try:
        return simple_eval(largest_ast, namespace)
    except ValueError:
        raise EvaluationError("Could not safely evaluate")


def evaluate_current_attribute(cursor_offset, line, namespace=None):
    """Safely evaluates the expression having an attributed accessed"""
    # this function runs user code in case of custom descriptors,
    # so could fail in any way

    obj = evaluate_current_expression(cursor_offset, line, namespace)
    attr = line_properties.current_expression_attribute(cursor_offset, line)
    if attr is None:
        raise EvaluationError("No attribute found to look up")
    try:
        return getattr(obj, attr.word)
    except AttributeError:
        raise EvaluationError(
            "can't lookup attribute %s on %r" % (attr.word, obj))


def safe_get_attribute(obj, attr):
    """Gets attributes without triggering descriptors on new-style classes"""
    if is_new_style(obj):
        with AttrCleaner(obj):
            result = safe_get_attribute_new_style(obj, attr)
            if isinstance(result, member_descriptor):
                # will either be the same slot descriptor or the value
                return getattr(obj, attr)
            return result
    return getattr(obj, attr)


class _ClassWithSlots(object):
    __slots__ = ['a']


member_descriptor = type(_ClassWithSlots.a)


def safe_get_attribute_new_style(obj, attr):
    """Returns approximately the attribute returned by getattr(obj, attr)

    The object returned ought to be callable if getattr(obj, attr) was.
    Fake callable objects may be returned instead, in order to avoid executing
    arbitrary code in descriptors.

    If the object is an instance of a class that uses __slots__, will return
    the member_descriptor object instead of the value.
    """
    if not is_new_style(obj):
        raise ValueError("%r is not a new-style class or object" % obj)
    to_look_through = (obj.__mro__
                       if inspect.isclass(obj)
                       else (obj,) + type(obj).__mro__)

    for cls in to_look_through:
        if hasattr(cls, '__dict__') and attr in cls.__dict__:
            return cls.__dict__[attr]

    raise AttributeError()



# encoding: utf-8

import re
import subprocess
import sys
import tempfile
from textwrap import dedent

from bpython import args
from bpython.test import (FixLanguageTestCase as TestCase, unittest)

try:
    from nose.plugins.attrib import attr
except ImportError:
    def attr(*args, **kwargs):
        def identity(func):
            return func
        return identity


@attr(speed='slow')
class TestExecArgs(unittest.TestCase):
    @unittest.skip("hangs under OpenBSD")
    def test_exec_dunder_file(self):
        with tempfile.NamedTemporaryFile(mode="w") as f:
            f.write(dedent("""\
                import sys
                sys.stderr.write(__file__)
                sys.stderr.flush()"""))
            f.flush()
            p = subprocess.Popen(
                [sys.executable] +
                (['-W', 'ignore'] if sys.version_info[:2] == (2, 6) else []) +
                ["-m", "bpython.curtsies", f.name],
                stderr=subprocess.PIPE,
                universal_newlines=True)
            (_, stderr) = p.communicate()

            self.assertEqual(stderr.strip(), f.name)

    @unittest.skip("hangs under OpenBSD")
    def test_exec_nonascii_file(self):
        with tempfile.NamedTemporaryFile(mode="w") as f:
            f.write(dedent('''\
                #!/usr/bin/env python
                # coding: utf-8
                "你好 # nonascii"
                '''))
            f.flush()
            try:
                subprocess.check_call([
                    sys.executable, '-m', 'bpython.curtsies',
                    f.name])
            except subprocess.CalledProcessError:
                self.fail('Error running module with nonascii characters')

    @unittest.skip("hangs under OpenBSD")
    def test_exec_nonascii_file_linenums(self):
        with tempfile.NamedTemporaryFile(mode="w") as f:
            f.write(dedent("""\
                #!/usr/bin/env python
                # coding: utf-8
                1/0
                """))
            f.flush()
            p = subprocess.Popen(
                [sys.executable, "-m", "bpython.curtsies",
                    f.name],
                stderr=subprocess.PIPE,
                universal_newlines=True)
            (_, stderr) = p.communicate()

            self.assertIn('line 3', clean_colors(stderr))


def clean_colors(s):
    return re.sub(r'\x1b[^m]*m', '', s)


class TestParse(TestCase):

    def test_version(self):
        with self.assertRaises(SystemExit):
            args.parse(['--version'])



# encoding: utf-8

# The MIT License
#
# Copyright (c) 2015 the bpython authors.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import pydoc

import bpython._internal
from bpython._py3compat import py3
from bpython.repl import getpreferredencoding


class NopPydocPager(object):
    def __enter__(self):
        self._orig_pager = pydoc.pager
        pydoc.pager = self

    def __exit__(self, *args):
        pydoc.pager = self._orig_pager

    def __call__(self, text):
        return None


class _Helper(bpython._internal._Helper):

    def __init__(self, repl=None):
        self._repl = repl
        pydoc.pager = self.pager

        super(_Helper, self).__init__()

    def pager(self, output):
        if not py3 and isinstance(output, str):
            output = output.decode(getpreferredencoding())
        self._repl.pager(output)

    def __call__(self, *args, **kwargs):
        if self._repl.reevaluating:
            with NopPydocPager():
                return super(_Helper, self).__call__(*args, **kwargs)
        else:
            return super(_Helper, self).__call__(*args, **kwargs)

# vim: sw=4 ts=4 sts=4 ai et



# encoding: utf-8

from __future__ import absolute_import

import linecache


class BPythonLinecache(dict):
    """Replaces the cache dict in the standard-library linecache module,
    to also remember (in an unerasable way) bpython console input."""

    def __init__(self, *args, **kwargs):
        super(BPythonLinecache, self).__init__(*args, **kwargs)
        self.bpython_history = []

    def is_bpython_filename(self, fname):
        try:
            return fname.startswith('<bpython-input-')
        except AttributeError:
            # In case the key isn't a string
            return False

    def get_bpython_history(self, key):
        """Given a filename provided by remember_bpython_input,
        returns the associated source string."""
        try:
            idx = int(key.split('-')[2][:-1])
            return self.bpython_history[idx]
        except (IndexError, ValueError):
            raise KeyError

    def remember_bpython_input(self, source):
        """Remembers a string of source code, and returns
        a fake filename to use to retrieve it later."""
        filename = '<bpython-input-%s>' % len(self.bpython_history)
        self.bpython_history.append((len(source), None,
                                     source.splitlines(True), filename))
        return filename

    def __getitem__(self, key):
        if self.is_bpython_filename(key):
            return self.get_bpython_history(key)
        return super(BPythonLinecache, self).__getitem__(key)

    def __contains__(self, key):
        if self.is_bpython_filename(key):
            try:
                self.get_bpython_history(key)
                return True
            except KeyError:
                return False
        return super(BPythonLinecache, self).__contains__(key)

    def __delitem__(self, key):
        if not self.is_bpython_filename(key):
            return super(BPythonLinecache, self).__delitem__(key)


def _bpython_clear_linecache():
    try:
        bpython_history = linecache.cache.bpython_history
    except AttributeError:
        bpython_history = []
    linecache.cache = BPythonLinecache()
    linecache.cache.bpython_history = bpython_history


# Monkey-patch the linecache module so that we're able
# to hold our command history there and have it persist
linecache.cache = BPythonLinecache(linecache.cache)
linecache.clearcache = _bpython_clear_linecache


def filename_for_console_input(code_string):
    """Remembers a string of source code, and returns
    a fake filename to use to retrieve it later."""
    try:
        return linecache.cache.remember_bpython_input(code_string)
    except AttributeError:
        # If someone else has patched linecache.cache, better for code to
        # simply be unavailable to inspect.getsource() than to raise
        # an exception.
        return '<input>'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import contextlib
import errno
import greenlet
import logging
import os
import re
import signal
import subprocess
import sys
import tempfile
import threading
import time
import unicodedata
from six.moves import range

from pygments import format
from bpython._py3compat import PythonLexer
from pygments.formatters import TerminalFormatter

import blessings

import curtsies
from curtsies import FSArray, fmtstr, FmtStr, Termmode
from curtsies import fmtfuncs
from curtsies import events

import bpython
from bpython.repl import Repl as BpythonRepl, SourceNotFound
from bpython.config import (Struct, loadini, default_config_path,
                            getpreferredencoding)
from bpython.formatter import BPythonFormatter
from bpython import autocomplete
from bpython.translations import _
from bpython._py3compat import py3, is_main_thread
from bpython.pager import get_pager_command

from bpython.curtsiesfrontend import replpainter as paint
from bpython.curtsiesfrontend import sitefix
from bpython.curtsiesfrontend.coderunner import CodeRunner, FakeOutput
from bpython.curtsiesfrontend.filewatch import ModuleChangedEventHandler
from bpython.curtsiesfrontend.interaction import StatusBar
from bpython.curtsiesfrontend.manual_readline import edit_keys
from bpython.curtsiesfrontend import events as bpythonevents
from bpython.curtsiesfrontend.parse import parse as bpythonparse
from bpython.curtsiesfrontend.parse import func_for_letter, color_for_letter
from bpython.curtsiesfrontend.preprocess import preprocess
from bpython.curtsiesfrontend.interpreter import (Interp,
                                                  code_finished_will_parse)

from curtsies.configfile_keynames import keymap as key_dispatch

if not py3:
    import imp
    import pkgutil


logger = logging.getLogger(__name__)

INCONSISTENT_HISTORY_MSG = "#<---History inconsistent with output shown--->"
CONTIGUITY_BROKEN_MSG = "#<---History contiguity broken by rewind--->"
HELP_MESSAGE = """
Thanks for using bpython!

See http://bpython-interpreter.org/ for more information and
http://docs.bpython-interpreter.org/ for docs.
Please report issues at https://github.com/bpython/bpython/issues

Features:
Try using undo ({config.undo_key})!
Edit the current line ({config.edit_current_block_key}) or the entire session ({config.external_editor_key}) in an external editor. (currently {config.editor})
Save sessions ({config.save_key}) or post them to pastebins ({config.pastebin_key})! Current pastebin helper: {config.pastebin_helper}
Reload all modules and rerun session ({config.reimport_key}) to test out changes to a module.
Toggle auto-reload mode ({config.toggle_file_watch_key}) to re-execute the current session when a module you've imported is modified.

bpython -i your_script.py runs a file in interactive mode
bpython -t your_script.py pastes the contents of a file into the session

A config file at {config_file_location} customizes keys and behavior of bpython.
You can also set which pastebin helper and which external editor to use.
See {example_config_url} for an example config file.
Press {config.edit_config_key} to edit this config file.
"""
EXAMPLE_CONFIG_URL = 'https://raw.githubusercontent.com/bpython/bpython/master/bpython/sample-config'
EDIT_SESSION_HEADER = """### current bpython session - make changes and save to reevaluate session.
### lines beginning with ### will be ignored.
### To return to bpython without reevaluating make no changes to this file
### or save an empty file.
"""

# more than this many events will be assumed to be a true paste event,
# i.e. control characters like '<Ctrl-a>' will be stripped
MAX_EVENTS_POSSIBLY_NOT_PASTE = 20

# This is needed for is_nop and should be removed once is_nop is fixed.
if py3:
    unicode = str


class FakeStdin(object):
    """The stdin object user code will reference

    In user code, sys.stdin.read() asks the user for interactive input,
    so this class returns control to the UI to get that input."""
    def __init__(self, coderunner, repl, configured_edit_keys=None):
        self.coderunner = coderunner
        self.repl = repl
        self.has_focus = False  # whether FakeStdin receives keypress events
        self.current_line = ''
        self.cursor_offset = 0
        self.old_num_lines = 0
        self.readline_results = []
        if configured_edit_keys:
            self.rl_char_sequences = configured_edit_keys
        else:
            self.rl_char_sequences = edit_keys

    def process_event(self, e):
        assert self.has_focus

        logger.debug('fake input processing event %r', e)
        if isinstance(e, events.PasteEvent):
            for ee in e.events:
                if ee not in self.rl_char_sequences:
                    self.add_input_character(ee)
        elif e in self.rl_char_sequences:
            self.cursor_offset, self.current_line = self.rl_char_sequences[e](
                self.cursor_offset, self.current_line)
        elif isinstance(e, events.SigIntEvent):
            self.coderunner.sigint_happened_in_main_context = True
            self.has_focus = False
            self.current_line = ''
            self.cursor_offset = 0
            self.repl.run_code_and_maybe_finish()
        elif e in ("<Esc+.>",):
            self.get_last_word()

        elif e in ["<ESC>"]:
            pass
        elif e in ['<Ctrl-d>']:
            if self.current_line == '':
                self.repl.send_to_stdin('\n')
                self.has_focus = False
                self.current_line = ''
                self.cursor_offset = 0
                self.repl.run_code_and_maybe_finish(for_code='')
            else:
                pass
        elif e in ["\n", "\r", "<Ctrl-j>", "<Ctrl-m>"]:
            line = self.current_line
            self.repl.send_to_stdin(line + '\n')
            self.has_focus = False
            self.current_line = ''
            self.cursor_offset = 0
            self.repl.run_code_and_maybe_finish(for_code=line + '\n')
        else:  # add normal character
            self.add_input_character(e)

        if self.current_line.endswith(("\n", "\r")):
            pass
        else:
            self.repl.send_to_stdin(self.current_line)

    def add_input_character(self, e):
        if e == '<SPACE>':
            e = ' '
        if e.startswith('<') and e.endswith('>'):
            return
        assert len(e) == 1, 'added multiple characters: %r' % e
        logger.debug('adding normal char %r to current line', e)

        c = e if py3 else e.encode('utf8')
        self.current_line = (self.current_line[:self.cursor_offset] +
                             c +
                             self.current_line[self.cursor_offset:])
        self.cursor_offset += 1

    def readline(self):
        self.has_focus = True
        self.repl.send_to_stdin(self.current_line)
        value = self.coderunner.request_from_main_context()
        self.readline_results.append(value)
        return value

    def readlines(self, size=-1):
        return list(iter(self.readline, ''))

    def __iter__(self):
        return iter(self.readlines())

    def isatty(self):
        return True

    def flush(self):
        """Flush the internal buffer. This is a no-op. Flushing stdin
        doesn't make any sense anyway."""

    def write(self, value):
        # XXX IPython expects sys.stdin.write to exist, there will no doubt be
        # others, so here's a hack to keep them happy
        raise IOError(errno.EBADF, "sys.stdin is read-only")

    def close(self):
        # hack to make closing stdin a nop
        # This is useful for multiprocessing.Process, which does work
        # for the most part, although output from other processes is
        # discarded.
        pass

    @property
    def encoding(self):
        return 'UTF8'

    # TODO write a read() method?


class ReevaluateFakeStdin(object):
    """Stdin mock used during reevaluation (undo) so raw_inputs don't have to
    be reentered"""
    def __init__(self, fakestdin, repl):
        self.fakestdin = fakestdin
        self.repl = repl
        self.readline_results = fakestdin.readline_results[:]

    def readline(self):
        if self.readline_results:
            value = self.readline_results.pop(0)
        else:
            value = 'no saved input available'
        self.repl.send_to_stdouterr(value)
        return value


class ImportLoader(object):

    def __init__(self, watcher, loader):
        self.watcher = watcher
        self.loader = loader

    def load_module(self, name):
        module = self.loader.load_module(name)
        if hasattr(module, '__file__'):
            self.watcher.track_module(module.__file__)
        return module


if not py3:
    # Remember that pkgutil.ImpLoader is an old style class.
    class ImpImportLoader(pkgutil.ImpLoader):

        def __init__(self, watcher, *args):
            self.watcher = watcher
            pkgutil.ImpLoader.__init__(self, *args)

        def load_module(self, name):
            module = pkgutil.ImpLoader.load_module(self, name)
            if hasattr(module, '__file__'):
                self.watcher.track_module(module.__file__)
            return module


class ImportFinder(object):

    def __init__(self, watcher, old_meta_path):
        self.watcher = watcher
        self.old_meta_path = old_meta_path

    def find_module(self, fullname, path=None):
        for finder in self.old_meta_path:
            loader = finder.find_module(fullname, path)
            if loader is not None:
                return ImportLoader(self.watcher, loader)

        if not py3:
            # Python 2 does not have the default finders stored in
            # sys.meta_path. Use imp to perform the actual importing.
            try:
                result = imp.find_module(fullname, path)
                return ImpImportLoader(self.watcher, fullname, *result)
            except ImportError:
                return None

        return None


class BaseRepl(BpythonRepl):
    """Python Repl

    Reacts to events like
     - terminal dimensions and change events
     - keystrokes
    Behavior altered by
     - number of scroll downs that were necessary to render array after each
       display
     - initial cursor position
    outputs:
     - 2D array to be rendered

    BaseRepl is mostly view-independent state of Repl - but self.width and
    self.height are important for figuring out how to wrap lines for example.
    Usually self.width and self.height should be set by receiving a window
    resize event, not manually set to anything - as long as the first event
    received is a window resize event, this works fine.

    Subclasses are responsible for implementing several methods.
    """

    def __init__(self,
                 locals_=None,
                 config=None,
                 banner=None,
                 interp=None,
                 orig_tcattrs=None):
        """
        locals_ is a mapping of locals to pass into the interpreter
        config is a bpython config.Struct with config attributes
        banner is a string to display briefly in the status bar
        interp is an interpreter instance to use
        original terminal state, useful for shelling out with normal terminal
        """

        logger.debug("starting init")

        if config is None:
            config = Struct()
            loadini(config, default_config_path())

        # If creating a new interpreter on undo would be unsafe because initial
        # state was passed in
        self.weak_rewind = bool(locals_ or interp)

        if interp is None:
            interp = Interp(locals=locals_)
            interp.write = self.send_to_stdouterr
        if banner is None:
            if config.help_key:
                banner = (_('Welcome to bpython!') + ' ' +
                          _('Press <%s> for help.') % config.help_key)
            else:
                banner = None
        # only one implemented currently
        config.autocomplete_mode = autocomplete.SIMPLE
        if config.cli_suggestion_width <= 0 or config.cli_suggestion_width > 1:
            config.cli_suggestion_width = 1

        self.reevaluating = False
        self.fake_refresh_requested = False

        self.status_bar = StatusBar('',
                                    request_refresh=self.request_refresh,
                                    schedule_refresh=self.schedule_refresh)
        self.edit_keys = edit_keys.mapping_with_config(config, key_dispatch)
        logger.debug("starting parent init")
        super(BaseRepl, self).__init__(interp, config)

        self.formatter = BPythonFormatter(config.color_scheme)

        # overwriting what bpython.Repl put there
        # interact is called to interact with the status bar,
        # so we're just using the same object
        self.interact = self.status_bar

        # line currently being edited, without ps1 (usually '>>> ')
        self._current_line = ''

        # current line of output - stdout and stdin go here
        self.current_stdouterr_line = ''

        # lines separated whenever logical line
        # length goes over what the terminal width
        # was at the time of original output
        self.display_lines = []

        # this is every line that's been executed; it gets smaller on rewind
        self.history = []

        # formatted version of lines in the buffer kept around so we can
        # unhighlight parens using self.reprint_line as called by bpython.Repl
        self.display_buffer = []

        # how many times display has been scrolled down
        # because there wasn't room to display everything
        self.scroll_offset = 0

        # from the left, 0 means first char
        self._cursor_offset = 0

        self.orig_tcattrs = orig_tcattrs

        self.coderunner = CodeRunner(self.interp, self.request_refresh)

        # filenos match the backing device for libs that expect it,
        # but writing to them will do weird things to the display
        self.stdout = FakeOutput(self.coderunner, self.send_to_stdouterr,
                                 fileno=sys.__stdout__.fileno())
        self.stderr = FakeOutput(self.coderunner, self.send_to_stdouterr,
                                 fileno=sys.__stderr__.fileno())
        self.stdin = FakeStdin(self.coderunner, self, self.edit_keys)

        # next paint should clear screen
        self.request_paint_to_clear_screen = False

        self.request_paint_to_pad_bottom = 0

        # offscreen command yields results different from scrollback buffer
        self.inconsistent_history = False

        # history error message has already been displayed
        self.history_already_messed_up = False

        # some commands act differently based on the prev event
        # this list doesn't include instances of event.Event,
        # only keypress-type events (no refresh screen events etc.)
        self.last_events = [None] * 50

        # displays prev events in a column on the right hand side
        self.presentation_mode = False

        self.paste_mode = False
        self.current_match = None
        self.list_win_visible = False
        # whether auto reloading active
        self.watching_files = config.default_autoreload

        # 'reverse_incremental_search', 'incremental_search' or None
        self.incr_search_mode = None

        self.incr_search_target = ''

        self.original_modules = set(sys.modules.keys())

        self.width = None
        self.height = None

        self.status_bar.message(banner)

        self.watcher = ModuleChangedEventHandler([], self.request_reload)
        if self.watcher and config.default_autoreload:
            self.watcher.activate()

    # The methods below should be overridden, but the default implementations
    # below can be used as well.

    def get_cursor_vertical_diff(self):
        """Return how the cursor moved due to a window size change"""
        return 0

    def get_top_usable_line(self):
        """Return the top line of display that can be rewritten"""
        return 0

    def get_term_hw(self):
        """Returns the current width and height of the display area."""
        return (50, 10)

    def _schedule_refresh(self, when='now'):
        """Arrange for the bpython display to be refreshed soon.

        This method will be called when the Repl wants the display to be
        refreshed at a known point in the future, and as such it should
        interrupt a pending request to the user for input.

        Because the worst-case effect of not refreshing
        is only having an out of date UI until the user enters input, a
        default NOP implementation is provided."""

    # The methods below must be overridden in subclasses.

    def _request_refresh(self):
        """Arrange for the bpython display to be refreshed soon.

        This method will be called when the Repl wants to refresh the display,
        but wants control returned to it afterwards. (it is assumed that simply
        returning from process_event will cause an event refresh)

        The very next event received by process_event should be a
        RefreshRequestEvent."""
        raise NotImplementedError

    def _request_reload(self, files_modified=('?',)):
        """Like request_refresh, but for reload requests events."""
        raise NotImplementedError

    def request_undo(self, n=1):
        """Like request_refresh, but for undo request events."""
        raise NotImplementedError

    def on_suspend(self):
        """Will be called on sigtstp.

        Do whatever cleanup would allow the user to use other programs."""
        raise NotImplementedError

    def after_suspend(self):
        """Will be called when process foregrounded after suspend.

        See to it that process_event is called with None to trigger a refresh
        if not in the middle of a process_event call when suspend happened."""
        raise NotImplementedError

    # end methods that should be overridden in subclass

    def request_refresh(self):
        """Request that the bpython display to be refreshed soon."""
        if self.reevaluating or self.paste_mode:
            self.fake_refresh_requested = True
        else:
            self._request_refresh()

    def request_reload(self, files_modified=()):
        """Request that a ReloadEvent be passed next into process_event"""
        if self.watching_files:
            self._request_reload(files_modified=files_modified)

    def schedule_refresh(self, when='now'):
        """Schedule a ScheduledRefreshRequestEvent for when.

        Such a event should interrupt if blockied waiting for keyboard input"""
        if self.reevaluating or self.paste_mode:
            self.fake_refresh_requested = True
        else:
            self._schedule_refresh(when=when)

    def __enter__(self):
        self.orig_stdout = sys.stdout
        self.orig_stderr = sys.stderr
        self.orig_stdin = sys.stdin
        sys.stdout = self.stdout
        sys.stderr = self.stderr
        sys.stdin = self.stdin
        self.orig_sigwinch_handler = signal.getsignal(signal.SIGWINCH)
        self.orig_sigtstp_handler = signal.getsignal(signal.SIGTSTP)

        if is_main_thread():
            # This turns off resize detection and ctrl-z suspension.
            signal.signal(signal.SIGWINCH, self.sigwinch_handler)
            signal.signal(signal.SIGTSTP, self.sigtstp_handler)

        self.orig_meta_path = sys.meta_path
        if self.watcher:
            sys.meta_path = [ImportFinder(self.watcher, self.orig_meta_path)]

        sitefix.monkeypatch_quit()
        return self

    def __exit__(self, *args):
        sys.stdin = self.orig_stdin
        sys.stdout = self.orig_stdout
        sys.stderr = self.orig_stderr

        if is_main_thread():
            # This turns off resize detection and ctrl-z suspension.
            signal.signal(signal.SIGWINCH, self.orig_sigwinch_handler)
            signal.signal(signal.SIGTSTP, self.orig_sigtstp_handler)

        sys.meta_path = self.orig_meta_path

    def sigwinch_handler(self, signum, frame):
        old_rows, old_columns = self.height, self.width
        self.height, self.width = self.get_term_hw()
        cursor_dy = self.get_cursor_vertical_diff()
        self.scroll_offset -= cursor_dy
        logger.info('sigwinch! Changed from %r to %r', (old_rows, old_columns),
                    (self.height, self.width))
        logger.info('decreasing scroll offset by %d to %d', cursor_dy,
                    self.scroll_offset)

    def sigtstp_handler(self, signum, frame):
        self.scroll_offset = len(self.lines_for_display)
        self.__exit__()
        self.on_suspend()
        os.kill(os.getpid(), signal.SIGTSTP)
        self.after_suspend()
        self.__enter__()

    def clean_up_current_line_for_exit(self):
        """Called when trying to exit to prep for final paint"""
        logger.debug('unhighlighting paren for exit')
        self.cursor_offset = -1
        self.unhighlight_paren()

    # Event handling
    def process_event(self, e):
        """Returns True if shutting down, otherwise returns None.
        Mostly mutates state of Repl object"""

        logger.debug("processing event %r", e)
        if isinstance(e, events.Event):
            return self.process_control_event(e)
        else:
            self.last_events.append(e)
            self.last_events.pop(0)
            return self.process_key_event(e)

    def process_control_event(self, e):

        if isinstance(e, bpythonevents.ScheduledRefreshRequestEvent):
            # This is a scheduled refresh - it's really just a refresh (so nop)
            pass

        elif isinstance(e, bpythonevents.RefreshRequestEvent):
            logger.info('received ASAP refresh request event')
            if self.status_bar.has_focus:
                self.status_bar.process_event(e)
            else:
                assert self.coderunner.code_is_waiting
                self.run_code_and_maybe_finish()

        elif self.status_bar.has_focus:
            return self.status_bar.process_event(e)

        # handles paste events for both stdin and repl
        elif isinstance(e, events.PasteEvent):
            ctrl_char = compress_paste_event(e)
            if ctrl_char is not None:
                return self.process_event(ctrl_char)
            with self.in_paste_mode():
                # Might not really be a paste, UI might just be lagging
                if (len(e.events) <= MAX_EVENTS_POSSIBLY_NOT_PASTE and
                        any(not is_simple_event(ee) for ee in e.events)):
                    for ee in e.events:
                        if self.stdin.has_focus:
                            self.stdin.process_event(ee)
                        else:
                            self.process_event(ee)
                else:
                    simple_events = just_simple_events(e.events)
                    source = preprocess(''.join(simple_events),
                                        self.interp.compile)
                    for ee in source:
                        if self.stdin.has_focus:
                            self.stdin.process_event(ee)
                        else:
                            self.process_simple_keypress(ee)

        elif isinstance(e, bpythonevents.RunStartupFileEvent):
            try:
                self.startup()
            except IOError as e:
                self.status_bar.message(
                    _('Executing PYTHONSTARTUP failed: %s') % (e, ))

        elif isinstance(e, bpythonevents.UndoEvent):
            self.undo(n=e.n)

        elif self.stdin.has_focus:
            return self.stdin.process_event(e)

        elif isinstance(e, events.SigIntEvent):
            logger.debug('received sigint event')
            self.keyboard_interrupt()
            return

        elif isinstance(e, bpythonevents.ReloadEvent):
            if self.watching_files:
                self.clear_modules_and_reevaluate()
                self.status_bar.message(
                    _('Reloaded at %s because %s modified.') % (
                        time.strftime('%X'),
                        ' & '.join(e.files_modified)))

        else:
            raise ValueError("Don't know how to handle event type: %r" % e)

    def process_key_event(self, e):
        # To find the curtsies name for a keypress, try
        # python -m curtsies.events
        if self.status_bar.has_focus:
            return self.status_bar.process_event(e)
        if self.stdin.has_focus:
            return self.stdin.process_event(e)

        if (e in (key_dispatch[self.config.right_key] +
                  key_dispatch[self.config.end_of_line_key] +
                  ("<RIGHT>",)) and
                self.config.curtsies_right_arrow_completion and
                self.cursor_offset == len(self.current_line)):

            self.current_line += self.current_suggestion
            self.cursor_offset = len(self.current_line)
        elif e in ("<UP>",) + key_dispatch[self.config.up_one_line_key]:
            self.up_one_line()
        elif e in ("<DOWN>",) + key_dispatch[self.config.down_one_line_key]:
            self.down_one_line()
        elif e in ("<Ctrl-d>",):
            self.on_control_d()
        elif e in ("<Ctrl-o>",):
            self.operate_and_get_next()
        elif e in ("<Esc+.>",):
            self.get_last_word()
        elif e in key_dispatch[self.config.reverse_incremental_search_key]:
            self.incremental_search(reverse=True)
        elif e in key_dispatch[self.config.incremental_search_key]:
            self.incremental_search()
        elif (e in (("<BACKSPACE>",) +
                    key_dispatch[self.config.backspace_key]) and
              self.incr_search_mode):
            self.add_to_incremental_search(self, backspace=True)
        elif e in self.edit_keys.cut_buffer_edits:
            self.readline_kill(e)
        elif e in self.edit_keys.simple_edits:
            self.cursor_offset, self.current_line = self.edit_keys.call(
                e,
                cursor_offset=self.cursor_offset,
                line=self.current_line,
                cut_buffer=self.cut_buffer)
        elif e in key_dispatch[self.config.cut_to_buffer_key]:
            self.cut_to_buffer()
        elif e in key_dispatch[self.config.reimport_key]:
            self.clear_modules_and_reevaluate()
        elif e in key_dispatch[self.config.toggle_file_watch_key]:
            return self.toggle_file_watch()
        elif e in key_dispatch[self.config.clear_screen_key]:
            self.request_paint_to_clear_screen = True
        elif e in key_dispatch[self.config.show_source_key]:
            self.show_source()
        elif e in key_dispatch[self.config.help_key]:
            self.pager(self.help_text())
        elif e in key_dispatch[self.config.exit_key]:
            raise SystemExit()
        elif e in ("\n", "\r", "<PADENTER>", "<Ctrl-j>", "<Ctrl-m>"):
            self.on_enter()
        elif e == '<TAB>':  # tab
            self.on_tab()
        elif e in ("<Shift-TAB>",):
            self.on_tab(back=True)
        elif e in key_dispatch[self.config.undo_key]:  # ctrl-r for undo
            self.prompt_undo()
        elif e in key_dispatch[self.config.save_key]:  # ctrl-s for save
            greenlet.greenlet(self.write2file).switch()
        elif e in key_dispatch[self.config.pastebin_key]:  # F8 for pastebin
            greenlet.greenlet(self.pastebin).switch()
        elif e in key_dispatch[self.config.copy_clipboard_key]:
            greenlet.greenlet(self.copy2clipboard).switch()
        elif e in key_dispatch[self.config.external_editor_key]:
            self.send_session_to_external_editor()
        elif e in key_dispatch[self.config.edit_config_key]:
            greenlet.greenlet(self.edit_config).switch()
        # TODO add PAD keys hack as in bpython.cli
        elif e in key_dispatch[self.config.edit_current_block_key]:
            self.send_current_block_to_external_editor()
        elif e in ["<ESC>"]:
            self.incr_search_mode = None
        elif e in ["<SPACE>"]:
            self.add_normal_character(' ')
        else:
            self.add_normal_character(e)

    def get_last_word(self):

        previous_word = _last_word(self.rl_history.entry)
        word = _last_word(self.rl_history.back())
        line = self.current_line
        self._set_current_line(line[:len(line) - len(previous_word)] + word,
                               reset_rl_history=False)
        self._set_cursor_offset(
            self.cursor_offset - len(previous_word) + len(word),
            reset_rl_history=False)

    def incremental_search(self, reverse=False, include_current=False):
        if self.incr_search_mode is None:
            self.rl_history.enter(self.current_line)
            self.incr_search_target = ''
        else:
            if self.incr_search_target:
                line = (self.rl_history.back(
                    False, search=True,
                    target=self.incr_search_target,
                    include_current=include_current)
                    if reverse else
                    self.rl_history.forward(
                        False, search=True,
                        target=self.incr_search_target,
                        include_current=include_current))
                self._set_current_line(line,
                                       reset_rl_history=False,
                                       clear_special_mode=False)
                self._set_cursor_offset(len(self.current_line),
                                        reset_rl_history=False,
                                        clear_special_mode=False)
        if reverse:
            self.incr_search_mode = 'reverse_incremental_search'
        else:
            self.incr_search_mode = 'incremental_search'

    def readline_kill(self, e):
        func = self.edit_keys[e]
        self.cursor_offset, self.current_line, cut = func(self.cursor_offset,
                                                          self.current_line)
        if self.last_events[-2] == e:  # consecutive kill commands accumulative
            if func.kills == 'ahead':
                self.cut_buffer += cut
            elif func.kills == 'behind':
                self.cut_buffer = cut + self.cut_buffer
            else:
                raise ValueError("cut value other than 'ahead' or 'behind'")
        else:
            self.cut_buffer = cut

    def on_enter(self, insert_into_history=True, reset_rl_history=True):
        # so the cursor isn't touching a paren TODO: necessary?
        self._set_cursor_offset(-1, update_completion=False)
        if reset_rl_history:
            self.rl_history.reset()

        self.history.append(self.current_line)
        self.push(self.current_line, insert_into_history=insert_into_history)

    def on_tab(self, back=False):
        """Do something on tab key
        taken from bpython.cli

        Does one of the following:
        1) add space to move up to the next %4==0 column
        2) complete the current word with characters common to all completions
        3) select the first or last match
        4) select the next or previous match if already have a match
        """

        def only_whitespace_left_of_cursor():
            """returns true if all characters before cursor are whitespace"""
            return not self.current_line[:self.cursor_offset].strip()

        logger.debug('self.matches_iter.matches:%r', self.matches_iter.matches)
        if only_whitespace_left_of_cursor():
            front_ws = (len(self.current_line[:self.cursor_offset]) -
                        len(self.current_line[:self.cursor_offset].lstrip()))
            to_add = 4 - (front_ws % self.config.tab_length)
            for unused in range(to_add):
                self.add_normal_character(' ')
            return

        # run complete() if we don't already have matches
        if len(self.matches_iter.matches) == 0:
            self.list_win_visible = self.complete(tab=True)

        # 3. check to see if we can expand the current word
        if self.matches_iter.is_cseq():
            cursor_and_line = self.matches_iter.substitute_cseq()
            self._cursor_offset, self._current_line = cursor_and_line
            # using _current_line so we don't trigger a completion reset
            if not self.matches_iter.matches:
                self.list_win_visible = self.complete()

        elif self.matches_iter.matches:
            self.current_match = (back and self.matches_iter.previous() or
                                  next(self.matches_iter))
            cursor_and_line = self.matches_iter.cur_line()
            self._cursor_offset, self._current_line = cursor_and_line
            # using _current_line so we don't trigger a completion reset
            self.list_win_visible = True

    def on_control_d(self):
        if self.current_line == '':
            raise SystemExit()
        else:
            self.current_line = (self.current_line[:self.cursor_offset] +
                                 self.current_line[(self.cursor_offset + 1):])

    def cut_to_buffer(self):
        self.cut_buffer = self.current_line[self.cursor_offset:]
        self.current_line = self.current_line[:self.cursor_offset]

    def yank_from_buffer(self):
        pass

    def operate_and_get_next(self):
        # If we have not navigated back in history
        # ctrl+o will have the same effect as enter
        self.on_enter(reset_rl_history=False)

    def up_one_line(self):
        self.rl_history.enter(self.current_line)
        self._set_current_line(tabs_to_spaces(self.rl_history.back(
            False,
            search=self.config.curtsies_right_arrow_completion)),
            update_completion=False,
            reset_rl_history=False)
        self._set_cursor_offset(len(self.current_line), reset_rl_history=False)

    def down_one_line(self):
        self.rl_history.enter(self.current_line)
        self._set_current_line(tabs_to_spaces(self.rl_history.forward(
            False,
            search=self.config.curtsies_right_arrow_completion)),
            update_completion=False,
            reset_rl_history=False)
        self._set_cursor_offset(len(self.current_line), reset_rl_history=False)

    def process_simple_keypress(self, e):
        # '\n' needed for pastes
        if e in ("<Ctrl-j>", "<Ctrl-m>", "<PADENTER>", "\n", "\r"):
            self.on_enter()
            while self.fake_refresh_requested:
                self.fake_refresh_requested = False
                self.process_event(bpythonevents.RefreshRequestEvent())
        elif isinstance(e, events.Event):
            pass  # ignore events
        elif e == '<SPACE>':
            self.add_normal_character(' ')
        else:
            self.add_normal_character(e)

    def send_current_block_to_external_editor(self, filename=None):
        text = self.send_to_external_editor(self.get_current_block())
        lines = [line for line in text.split('\n')]
        while lines and not lines[-1].split():
            lines.pop()
        events = '\n'.join(lines + ([''] if len(lines) == 1 else ['', '']))
        self.clear_current_block()
        with self.in_paste_mode():
            for e in events:
                self.process_simple_keypress(e)
        self.cursor_offset = len(self.current_line)

    def send_session_to_external_editor(self, filename=None):
        for_editor = EDIT_SESSION_HEADER
        for_editor += '\n'.join(line[len(self.ps1):]
                                if line.startswith(self.ps1) else
                                line[len(self.ps2):]
                                if line.startswith(self.ps2) else
                                '### ' + line
                                for line in self.getstdout().split('\n'))
        text = self.send_to_external_editor(for_editor)
        if text == for_editor:
            self.status_bar.message(
                _('Session not reevaluated because it was not edited'))
            return
        lines = text.split('\n')
        if not lines[-1].strip():
            lines.pop()  # strip last line if empty
        if lines[-1].startswith('### '):
            current_line = lines[-1][4:]
        else:
            current_line = ''
        from_editor = [line for line in lines if line[:3] != '###']
        if all(not line.strip() for line in from_editor):
            self.status_bar.message(
                _('Session not reevaluated because saved file was blank'))
            return

        source = preprocess('\n'.join(from_editor), self.interp.compile)
        lines = source.split('\n')
        self.history = lines
        self.reevaluate(insert_into_history=True)
        self.current_line = current_line
        self.cursor_offset = len(self.current_line)
        self.status_bar.message(_('Session edited and reevaluated'))

    def clear_modules_and_reevaluate(self):
        if self.watcher:
            self.watcher.reset()
        cursor, line = self.cursor_offset, self.current_line
        for modname in (set(sys.modules.keys()) - self.original_modules):
            del sys.modules[modname]
        self.reevaluate(insert_into_history=True)
        self.cursor_offset, self.current_line = cursor, line
        self.status_bar.message(_('Reloaded at %s by user.') %
                                (time.strftime('%X'), ))

    def toggle_file_watch(self):
        if self.watcher:
            if self.watching_files:
                msg = _("Auto-reloading deactivated.")
                self.status_bar.message(msg)
                self.watcher.deactivate()
                self.watching_files = False
            else:
                msg = _("Auto-reloading active, watching for file changes...")
                self.status_bar.message(msg)
                self.watching_files = True
                self.watcher.activate()
        else:
            self.status_bar.message(_('Auto-reloading not available because '
                                      'watchdog not installed.'))

    # Handler Helpers
    def add_normal_character(self, char):
        if len(char) > 1 or is_nop(char):
            return
        if self.incr_search_mode:
            self.add_to_incremental_search(char)
        else:
            self._set_current_line((self.current_line[:self.cursor_offset] +
                                    char +
                                    self.current_line[self.cursor_offset:]),
                                   update_completion=False,
                                   reset_rl_history=False,
                                   clear_special_mode=False)
            self.cursor_offset += 1
        if (self.config.cli_trim_prompts and
                self.current_line.startswith(self.ps1)):
            self.current_line = self.current_line[4:]
            self.cursor_offset = max(0, self.cursor_offset - 4)

    def add_to_incremental_search(self, char=None, backspace=False):
        """Modify the current search term while in incremental search.

        The only operations allowed in incremental search mode are
        adding characters and backspacing."""
        if char is None and not backspace:
            raise ValueError("must provide a char or set backspace to True")
        if backspace:
            self.incr_search_target = self.incr_search_target[:-1]
        else:
            self.incr_search_target += char
        if self.incr_search_mode == 'reverse_incremental_search':
            self.incremental_search(reverse=True, include_current=True)
        elif self.incr_search_mode == 'incremental_search':
            self.incremental_search(include_current=True)
        else:
            raise ValueError('add_to_incremental_search not in a special mode')

    def update_completion(self, tab=False):
        """Update visible docstring and matches and box visibility"""
        # Update autocomplete info; self.matches_iter and self.funcprops
        # Should be called whenever the completion box might need to appear
        # or disappear; whenever current line or cursor offset changes,
        # unless this happened via selecting a match
        self.current_match = None
        self.list_win_visible = self.complete(tab)

    def predicted_indent(self, line):
        # TODO get rid of this! It's repeated code! Combine with Repl.
        logger.debug('line is %r', line)
        indent = len(re.match(r'[ ]*', line).group())
        if line.endswith(':'):
            indent = max(0, indent + self.config.tab_length)
        elif line and line.count(' ') == len(line):
            indent = max(0, indent - self.config.tab_length)
        elif (line and ':' not in line and line.strip().startswith(
                ('return', 'pass', 'raise', 'yield'))):
            indent = max(0, indent - self.config.tab_length)
        logger.debug('indent we found was %s', indent)
        return indent

    def push(self, line, insert_into_history=True):
        """Push a line of code onto the buffer, start running the buffer

        If the interpreter successfully runs the code, clear the buffer
        """
        if self.paste_mode:
            self.saved_indent = 0
        else:
            self.saved_indent = self.predicted_indent(line)

        if self.config.syntax:
            display_line = bpythonparse(format(
                self.tokenize(line), self.formatter))
            # self.tokenize requires that the line not be in self.buffer yet

            logger.debug('display line being pushed to buffer: %r -> %r',
                         line, display_line)
            self.display_buffer.append(display_line)
        else:
            self.display_buffer.append(fmtstr(line))

        if insert_into_history:
            self.insert_into_history(line)
        self.buffer.append(line)

        code_to_run = '\n'.join(self.buffer)

        logger.debug('running %r in interpreter', self.buffer)
        c, code_will_parse = code_finished_will_parse('\n'.join(self.buffer),
                                                      self.interp.compile)
        self.saved_predicted_parse_error = not code_will_parse
        if c:
            logger.debug('finished - buffer cleared')
            self.cursor_offset = 0
            self.display_lines.extend(self.display_buffer_lines)
            self.display_buffer = []
            self.buffer = []

        self.coderunner.load_code(code_to_run)
        self.run_code_and_maybe_finish()

    def run_code_and_maybe_finish(self, for_code=None):
        r = self.coderunner.run_code(for_code=for_code)
        if r:
            logger.debug("----- Running finish command stuff -----")
            logger.debug("saved_indent: %r", self.saved_indent)
            err = self.saved_predicted_parse_error
            self.saved_predicted_parse_error = False

            indent = self.saved_indent
            if err:
                indent = 0

            if self.rl_history.index == 0:
                self._set_current_line(' ' * indent, update_completion=True)
            else:
                self._set_current_line(self.rl_history.entries[-self.rl_history.index],
                                       reset_rl_history=False)
            self.cursor_offset = len(self.current_line)

    def keyboard_interrupt(self):
        # TODO factor out the common cleanup from running a line
        self.cursor_offset = -1
        self.unhighlight_paren()
        self.display_lines.extend(self.display_buffer_lines)
        self.display_lines.extend(paint.display_linize(
            self.current_cursor_line, self.width))
        self.display_lines.extend(paint.display_linize(
            "KeyboardInterrupt", self.width))
        self.clear_current_block(remove_from_history=False)

    def unhighlight_paren(self):
        """Modify line in self.display_buffer to unhighlight a paren if
        possible.

        self.highlighted_paren should be a line in ?
        """
        if self.highlighted_paren is not None and self.config.syntax:
            lineno, saved_tokens = self.highlighted_paren
            if lineno == len(self.display_buffer):
                # then this is the current line, so don't worry about it
                return
            self.highlighted_paren = None
            logger.debug('trying to unhighlight a paren on line %r', lineno)
            logger.debug('with these tokens: %r', saved_tokens)
            new = bpythonparse(format(saved_tokens, self.formatter))
            self.display_buffer[lineno] = self.display_buffer[lineno] \
                .setslice_with_length(0, len(new), new,
                                      len(self.display_buffer[lineno]))

    def clear_current_block(self, remove_from_history=True):
        self.display_buffer = []
        if remove_from_history:
            for unused in self.buffer:
                self.history.pop()
        self.buffer = []
        self.cursor_offset = 0
        self.saved_indent = 0
        self.current_line = ''
        self.cursor_offset = len(self.current_line)

    def get_current_block(self):
        return '\n'.join(self.buffer + [self.current_line])

    def send_to_stdouterr(self, output):
        """Send unicode strings or FmtStr to Repl stdout or stderr

        Must be able to handle FmtStrs because interpreter pass in
        tracebacks already formatted."""
        if not output:
            return
        lines = output.split('\n')
        logger.debug('display_lines: %r', self.display_lines)
        self.current_stdouterr_line += lines[0]
        if len(lines) > 1:
            self.display_lines.extend(paint.display_linize(
                self.current_stdouterr_line, self.width, blank_line=True))
            self.display_lines.extend(
                sum((paint.display_linize(line, self.width,
                                          blank_line=True)
                     for line in lines[1:-1]), []))
            self.current_stdouterr_line = lines[-1]
        logger.debug('display_lines: %r', self.display_lines)

    def send_to_stdin(self, line):
        if line.endswith('\n'):
            self.display_lines.extend(
                paint.display_linize(self.current_output_line, self.width))
            self.current_output_line = ''

    # formatting, output
    @property
    def done(self):
        """Whether the last block is complete - which prompt to use, ps1 or
        ps2"""
        return not self.buffer

    @property
    def current_line_formatted(self):
        """The colored current line (no prompt, not wrapped)"""
        if self.config.syntax:
            fs = bpythonparse(format(self.tokenize(self.current_line),
                              self.formatter))
            if self.incr_search_mode:
                if self.incr_search_target in self.current_line:
                    fs = fmtfuncs.on_magenta(self.incr_search_target).join(
                        fs.split(self.incr_search_target))
            elif (self.rl_history.saved_line and
                    self.rl_history.saved_line in self.current_line):
                if (self.config.curtsies_right_arrow_completion and
                        self.rl_history.index != 0):
                    fs = fmtfuncs.on_magenta(self.rl_history.saved_line).join(
                        fs.split(self.rl_history.saved_line))
            logger.debug('Display line %r -> %r', self.current_line, fs)
        else:
            fs = fmtstr(self.current_line)
        if hasattr(self, 'old_fs') and str(fs) != str(self.old_fs):
            pass
        self.old_fs = fs
        return fs

    @property
    def lines_for_display(self):
        """All display lines (wrapped, colored, with prompts)"""
        return self.display_lines + self.display_buffer_lines

    @property
    def display_buffer_lines(self):
        """The display lines (wrapped, colored, +prompts) of current buffer"""
        lines = []
        for display_line in self.display_buffer:
            prompt = func_for_letter(self.config.color_scheme['prompt'])
            more = func_for_letter(self.config.color_scheme['prompt_more'])
            display_line = ((more(self.ps2) if lines else prompt(self.ps1)) +
                            display_line)
            for line in paint.display_linize(display_line, self.width):
                lines.append(line)
        return lines

    @property
    def display_line_with_prompt(self):
        """colored line with prompt"""
        prompt = func_for_letter(self.config.color_scheme['prompt'])
        more = func_for_letter(self.config.color_scheme['prompt_more'])
        if self.incr_search_mode == 'reverse_incremental_search':
            return (prompt('(reverse-i-search)`{}\': '.format(
                self.incr_search_target)) + self.current_line_formatted)
        elif self.incr_search_mode == 'incremental_search':
            return (prompt('(i-search)`%s\': '.format(
                self.incr_search_target)) + self.current_line_formatted)
        return ((prompt(self.ps1) if self.done else more(self.ps2)) +
                self.current_line_formatted)

    @property
    def current_cursor_line_without_suggestion(self):
        """Current line, either output/input or Python prompt + code"""
        value = (self.current_output_line +
                 ('' if self.coderunner.running else
                  self.display_line_with_prompt))
        logger.debug('current cursor line: %r', value)
        return value

    @property
    def current_cursor_line(self):
        if self.config.curtsies_right_arrow_completion:
            suggest = func_for_letter(
                self.config.color_scheme['right_arrow_suggestion'])
            return (self.current_cursor_line_without_suggestion +
                    suggest(self.current_suggestion))
        else:
            return self.current_cursor_line_without_suggestion

    @property
    def current_suggestion(self):
        if self.current_line:
            for entry in reversed(self.rl_history.entries):
                if entry.startswith(self.current_line):
                    return entry[len(self.current_line):]
        return ''

    @property
    def current_output_line(self):
        """line of output currently being written, and stdin typed"""
        return self.current_stdouterr_line + self.stdin.current_line

    @current_output_line.setter
    def current_output_line(self, value):
        self.current_stdouterr_line = ''
        self.stdin.current_line = '\n'

    def paint(self, about_to_exit=False, user_quit=False,
              try_preserve_history_height=30,
              min_infobox_height=5):
        """Returns an array of min_height or more rows and width columns, plus
        cursor position

        Paints the entire screen - ideally the terminal display layer will take
        a diff and only write to the screen in portions that have changed, but
        the idea is that we don't need to worry about that here, instead every
        frame is completely redrawn because less state is cool!

        try_preserve_history_height is the the number of rows of content that
        must be visible before the suggestion box scrolls the terminal in order
        to display more than min_infobox_height rows of suggestions, docs etc.
        """
        # The hairiest function in the curtsies
        if about_to_exit:
            # exception to not changing state!
            self.clean_up_current_line_for_exit()

        width, min_height = self.width, self.height
        show_status_bar = ((bool(self.status_bar.should_show_message) or
                            self.status_bar.has_focus) and
                           not self.request_paint_to_pad_bottom)
        if show_status_bar:
            # because we're going to tack the status bar on at the end, shoot
            # for an array one less than the height of the screen
            min_height -= 1

        current_line_start_row = (len(self.lines_for_display) -
                                  max(0, self.scroll_offset))
        # TODO how is the situation of self.scroll_offset < 0 possible?
        # or show_status_bar and about_to_exit ?
        if self.request_paint_to_clear_screen:
            self.request_paint_to_clear_screen = False
            arr = FSArray(min_height + current_line_start_row, width)
        elif self.request_paint_to_pad_bottom:
            # min_height - 1 for startup banner with python version
            height = min(self.request_paint_to_pad_bottom, min_height - 1)
            arr = FSArray(height, width)
            self.request_paint_to_pad_bottom = 0
        else:
            arr = FSArray(0, width)
        # TODO test case of current line filling up the whole screen (there
        # aren't enough rows to show it)

        current_line = paint.paint_current_line(min_height, width,
                                                self.current_cursor_line)
        # needs to happen before we calculate contents of history because
        # calculating self.current_cursor_line has the side effect of
        # unhighlighting parens in buffer

        def move_screen_up(current_line_start_row):
            # move screen back up a screen minus a line
            while current_line_start_row < 0:
                logger.debug('scroll_offset was %s, current_line_start_row '
                             'was %s', self.scroll_offset,
                             current_line_start_row)
                self.scroll_offset = self.scroll_offset - self.height
                current_line_start_row = (len(self.lines_for_display) -
                                          max(-1, self.scroll_offset))
                logger.debug('scroll_offset changed to %s, '
                             'current_line_start_row changed to %s',
                             self.scroll_offset, current_line_start_row)
            return current_line_start_row

        if self.inconsistent_history and not self.history_already_messed_up:
            logger.debug(INCONSISTENT_HISTORY_MSG)
            self.history_already_messed_up = True
            msg = INCONSISTENT_HISTORY_MSG
            arr[0, 0:min(len(msg), width)] = [msg[:width]]
            current_line_start_row += 1  # for the message

            # to make up for the scroll that will be received after the
            # scrolls are rendered down a line
            self.scroll_offset -= 1

            current_line_start_row = move_screen_up(current_line_start_row)
            logger.debug('current_line_start_row: %r', current_line_start_row)

            history = paint.paint_history(max(0, current_line_start_row - 1),
                                          width, self.lines_for_display)
            arr[1:history.height + 1, :history.width] = history

            if arr.height <= min_height:
                # force scroll down to hide broken history message
                arr[min_height, 0] = ' '

        elif current_line_start_row < 0:
            # if current line trying to be drawn off the top of the screen
            logger.debug(CONTIGUITY_BROKEN_MSG)
            msg = CONTIGUITY_BROKEN_MSG
            arr[0, 0:min(len(msg), width)] = [msg[:width]]

            current_line_start_row = move_screen_up(current_line_start_row)

            history = paint.paint_history(max(0, current_line_start_row - 1),
                                          width, self.lines_for_display)
            arr[1:history.height + 1, :history.width] = history

            if arr.height <= min_height:
                # force scroll down to hide broken history message
                arr[min_height, 0] = ' '

        else:
            assert current_line_start_row >= 0
            logger.debug("no history issues. start %i", current_line_start_row)
            history = paint.paint_history(current_line_start_row, width,
                                          self.lines_for_display)
            arr[:history.height, :history.width] = history

        self.inconsistent_history = False

        if user_quit:  # quit() or exit() in interp
            current_line_start_row = (current_line_start_row -
                                      current_line.height)
        logger.debug("---current line row slice %r, %r",
                     current_line_start_row,
                     current_line_start_row + current_line.height)
        logger.debug("---current line col slice %r, %r", 0, current_line.width)
        arr[current_line_start_row:(current_line_start_row +
                                    current_line.height),
            0:current_line.width] = current_line

        if current_line.height > min_height:
            return arr, (0, 0)  # short circuit, no room for infobox

        lines = paint.display_linize(self.current_cursor_line + 'X', width)
        # extra character for space for the cursor
        current_line_end_row = current_line_start_row + len(lines) - 1
        current_line_height = current_line_end_row - current_line_start_row

        if self.stdin.has_focus:
            cursor_row, cursor_column = divmod(
                    len(self.current_stdouterr_line) +
                    self.stdin.cursor_offset, width)
            assert cursor_column >= 0, cursor_column
        elif self.coderunner.running:  # TODO does this ever happen?
            cursor_row, cursor_column = divmod(
                    (len(self.current_cursor_line_without_suggestion) +
                     self.cursor_offset),
                    width)
            assert cursor_column >= 0, (
                       cursor_column, len(self.current_cursor_line),
                       len(self.current_line), self.cursor_offset)
        else:
            cursor_row, cursor_column = divmod(
                    (len(self.current_cursor_line_without_suggestion) -
                     len(self.current_line) + self.cursor_offset),
                    width)
            assert cursor_column >= 0, (
                       cursor_column, len(self.current_cursor_line),
                       len(self.current_line), self.cursor_offset)
        cursor_row += current_line_start_row

        if self.list_win_visible and not self.coderunner.running:
            logger.debug('infobox display code running')
            visible_space_above = history.height
            potential_space_below = min_height - current_line_end_row - 1
            visible_space_below = (potential_space_below -
                                   self.get_top_usable_line())

            if self.config.curtsies_list_above:
                info_max_rows = max(visible_space_above, visible_space_below)
            else:
                # Logic for determining size of completion box
                # smallest allowed over-full completion box
                preferred_height = max(
                        # always make infobox at least this height
                        min_infobox_height,

                        # use this value if there's so much space that we can
                        # preserve this try_preserve_history_height rows history
                        min_height - try_preserve_history_height)

                info_max_rows = min(max(visible_space_below,
                                        preferred_height),
                                    min_height - current_line_height - 1)
            infobox = paint.paint_infobox(
                    info_max_rows,
                    int(width * self.config.cli_suggestion_width),
                    self.matches_iter.matches,
                    self.funcprops,
                    self.arg_pos,
                    self.current_match,
                    self.docstring,
                    self.config,
                    self.matches_iter.completer.format
                    if self.matches_iter.completer else None)

            if (visible_space_below >= infobox.height or
                    not self.config.curtsies_list_above):
                arr[current_line_end_row + 1:(current_line_end_row + 1 +
                                              infobox.height),
                    0:infobox.width] = infobox
            else:
                arr[current_line_start_row - infobox.height:
                    current_line_start_row, 0:infobox.width] = infobox
                logger.debug('infobox of shape %r added to arr of shape %r',
                             infobox.shape, arr.shape)

        logger.debug('about to exit: %r', about_to_exit)
        if show_status_bar:
            statusbar_row = (min_height
                             if arr.height == min_height else arr.height)
            if about_to_exit:
                arr[statusbar_row, :] = FSArray(1, width)
            else:
                arr[statusbar_row, :] = paint.paint_statusbar(
                    1, width, self.status_bar.current_line, self.config)

        if self.presentation_mode:
            rows = arr.height
            columns = arr.width
            last_key_box = paint.paint_last_events(
                    rows, columns,
                    [events.pp_event(x) for x in self.last_events if x],
                    self.config)
            arr[arr.height - last_key_box.height:arr.height,
                arr.width - last_key_box.width:arr.width] = last_key_box

        if self.config.color_scheme['background'] not in ('d', 'D'):
            for r in range(arr.height):
                bg = color_for_letter(self.config.color_scheme['background'])
                arr[r] = fmtstr(arr[r], bg=bg)
        logger.debug('returning arr of size %r', arr.shape)
        logger.debug('cursor pos: %r', (cursor_row, cursor_column))
        return arr, (cursor_row, cursor_column)

    @contextlib.contextmanager
    def in_paste_mode(self):
        orig_value = self.paste_mode
        self.paste_mode = True
        yield
        self.paste_mode = orig_value
        if not self.paste_mode:
            self.update_completion()

    def __repr__(self):
        s = ''
        s += '<' + repr(type(self)) + '\n'
        s += " cursor_offset:" + repr(self.cursor_offset) + '\n'
        s += " num display lines:" + repr(len(self.display_lines)) + '\n'
        s += " lines scrolled down:" + repr(self.scroll_offset) + '\n'
        s += '>'
        return s

    def _get_current_line(self):
        return self._current_line

    def _set_current_line(self, line, update_completion=True,
                          reset_rl_history=True, clear_special_mode=True):
        if self._current_line == line:
            return
        self._current_line = line
        if self.paste_mode:
            return
        if update_completion:
            self.update_completion()
        if reset_rl_history:
            self.rl_history.reset()
        if clear_special_mode:
            self.special_mode = None
        self.unhighlight_paren()

    current_line = property(_get_current_line, _set_current_line, None,
                            "The current line")

    def _get_cursor_offset(self):
        return self._cursor_offset

    def _set_cursor_offset(self, offset, update_completion=True,
                           reset_rl_history=False, clear_special_mode=True):
        if self._cursor_offset == offset:
            return
        if self.paste_mode:
            self._cursor_offset = offset
            self.unhighlight_paren()
            return
        if reset_rl_history:
            self.rl_history.reset()
        if clear_special_mode:
            self.incr_search_mode = None
        self._cursor_offset = offset
        if update_completion:
            self.update_completion()
        self.unhighlight_paren()

    cursor_offset = property(_get_cursor_offset, _set_cursor_offset, None,
                             "The current cursor offset from the front of the "
                             "line")

    def echo(self, msg, redraw=True):
        """
        Notification that redrawing the current line is necessary (we don't
        care, since we always redraw the whole screen)

        Supposed to parse and echo a formatted string with appropriate
        attributes. It's not supposed to update the screen if it's reevaluating
        the code (as it does with undo)."""
        logger.debug("echo called with %r" % msg)

    @property
    def cpos(self):
        "many WATs were had - it's the pos from the end of the line back"""
        return len(self.current_line) - self.cursor_offset

    def reprint_line(self, lineno, tokens):
        logger.debug("calling reprint line with %r %r", lineno, tokens)
        if self.config.syntax:
            self.display_buffer[lineno] = bpythonparse(
                    format(tokens, self.formatter))

    def take_back_buffer_line(self):
        assert len(self.buffer) > 0
        if len(self.buffer) == 1:
            self._cursor_offset = 0
            self.current_line = ''
        else:
            line = self.buffer[-1]
            indent = self.predicted_indent(line)
            self._current_line = indent * ' '
            self.cursor_offset = len(self.current_line)
        self.display_buffer.pop()
        self.buffer.pop()
        self.history.pop()

    def take_back_empty_line(self):
        assert self.history and not self.history[-1]
        self.history.pop()
        self.display_lines.pop()

    def prompt_undo(self):
        if self.buffer:
            return self.take_back_buffer_line()
        if self.history and not self.history[-1]:
            return self.take_back_empty_line()

        def prompt_for_undo():
            n = BpythonRepl.prompt_undo(self)
            if n > 0:
                self.request_undo(n=n)

        greenlet.greenlet(prompt_for_undo).switch()

    def reevaluate(self, insert_into_history=False):
        """bpython.Repl.undo calls this"""
        if self.watcher:
            self.watcher.reset()
        old_logical_lines = self.history
        old_display_lines = self.display_lines
        self.history = []
        self.display_lines = []

        if not self.weak_rewind:
            self.interp = self.interp.__class__()
            self.interp.write = self.send_to_stdouterr
            self.coderunner.interp = self.interp
            self.initialize_interp()

        self.buffer = []
        self.display_buffer = []
        self.highlighted_paren = None

        self.process_event(bpythonevents.RunStartupFileEvent())
        self.reevaluating = True
        sys.stdin = ReevaluateFakeStdin(self.stdin, self)
        for line in old_logical_lines:
            self._current_line = line
            self.on_enter(insert_into_history=insert_into_history)
            while self.fake_refresh_requested:
                self.fake_refresh_requested = False
                self.process_event(bpythonevents.RefreshRequestEvent())
        sys.stdin = self.stdin
        self.reevaluating = False

        num_lines_onscreen = (len(self.lines_for_display) -
                              max(0, self.scroll_offset))
        display_lines_offscreen = self.display_lines[:len(self.display_lines) -
                                                     num_lines_onscreen]
        old_display_lines_offscreen = old_display_lines[:(
            len(self.display_lines) - num_lines_onscreen)]
        logger.debug('old_display_lines_offscreen %s', '|'.join(
            str(x) for x in old_display_lines_offscreen))
        logger.debug('    display_lines_offscreen %s', '|'.join(
            str(x) for x in display_lines_offscreen))
        if ((old_display_lines_offscreen[:len(display_lines_offscreen)] !=
                display_lines_offscreen) and
                not self.history_already_messed_up):
            self.inconsistent_history = True
        logger.debug('after rewind, self.inconsistent_history is %r',
                     self.inconsistent_history)

        self._cursor_offset = 0
        self.current_line = ''

    def initialize_interp(self):
        self.coderunner.interp.locals['_repl'] = self
        self.coderunner.interp.runsource(
            'from bpython.curtsiesfrontend._internal '
            'import _Helper')
        self.coderunner.interp.runsource('help = _Helper(_repl)\n')

        del self.coderunner.interp.locals['_repl']
        del self.coderunner.interp.locals['_Helper']

    def getstdout(self):
        lines = self.lines_for_display + [self.current_line_formatted]
        s = '\n'.join(x.s if isinstance(x, FmtStr) else x for x in lines) \
            if lines else ''
        return s

    def focus_on_subprocess(self, args):
        prev_sigwinch_handler = signal.getsignal(signal.SIGWINCH)
        try:
            signal.signal(signal.SIGWINCH, self.orig_sigwinch_handler)
            with Termmode(self.orig_stdin, self.orig_tcattrs):
                terminal = blessings.Terminal(stream=sys.__stdout__)
                with terminal.fullscreen():
                    sys.__stdout__.write(terminal.save)
                    sys.__stdout__.write(terminal.move(0, 0))
                    sys.__stdout__.flush()
                    p = subprocess.Popen(args,
                                         stdin=self.orig_stdin,
                                         stderr=sys.__stderr__,
                                         stdout=sys.__stdout__)
                    p.wait()
                    sys.__stdout__.write(terminal.restore)
                    sys.__stdout__.flush()
        finally:
            signal.signal(signal.SIGWINCH, prev_sigwinch_handler)

    def pager(self, text):
        """Runs an external pager on text

        text must be a unicode"""
        command = get_pager_command()
        with tempfile.NamedTemporaryFile() as tmp:
            tmp.write(text.encode(getpreferredencoding()))
            tmp.flush()
            self.focus_on_subprocess(command + [tmp.name])

    def show_source(self):
        try:
            source = self.get_source_of_current_name()
        except SourceNotFound as e:
            self.status_bar.message('%s' % (e, ))
        else:
            if self.config.highlight_show_source:
                source = format(PythonLexer().get_tokens(source),
                                TerminalFormatter())
            self.pager(source)

    def help_text(self):
        return self.version_help_text() + '\n' + self.key_help_text()

    def version_help_text(self):
        return (('bpython-curtsies version %s' % bpython.__version__) + ' ' +
                ('using curtsies version %s' % curtsies.__version__) + '\n' +
                HELP_MESSAGE.format(config_file_location=default_config_path(),
                                    example_config_url=EXAMPLE_CONFIG_URL,
                                    config=self.config))

    def key_help_text(self):
        NOT_IMPLEMENTED = ('suspend', 'cut to buffer', 'search', 'last output',
                           'yank from buffer', 'cut to buffer')
        pairs = []
        pairs.append(['complete history suggestion',
                      'right arrow at end of line'])
        pairs.append(['previous match with current line', 'up arrow'])
        for functionality, key in [(attr[:-4].replace('_', ' '),
                                    getattr(self.config, attr))
                                   for attr in self.config.__dict__
                                   if attr.endswith('key')]:
            if functionality in NOT_IMPLEMENTED:
                key = 'Not Implemented'
            if key == '':
                key = 'Disabled'

            pairs.append([functionality, key])

        max_func = max(len(func) for func, key in pairs)
        return '\n'.join('%s : %s' % (func.rjust(max_func), key)
                         for func, key in pairs)


def is_nop(char):
    return unicodedata.category(unicode(char)) == 'Cc'


def tabs_to_spaces(line):
    return line.replace('\t', '    ')


def _last_word(line):
    return line.split().pop() if line.split() else ''


def compress_paste_event(paste_event):
    """If all events in a paste event are identical and not simple characters,
    returns one of them

    Useful for when the UI is running so slowly that repeated keypresses end up
    in a paste event.  If we value not getting delayed and assume the user is
    holding down a key to produce such frequent key events, it makes sense to
    drop some of the events.
    """
    if not all(paste_event.events[0] == e for e in paste_event.events):
        return None
    event = paste_event.events[0]
    # basically "is there a special curtsies names for this key?"
    if len(event) > 1:
        return event
    else:
        return None


def just_simple_events(event_list):
    simple_events = []
    for e in event_list:
        # '\n' necessary for pastes
        if e in ("<Ctrl-j>", "<Ctrl-m>", "<PADENTER>", "\n", "\r"):
            simple_events.append('\n')
        elif isinstance(e, events.Event):
            pass  # ignore events
        elif e == '<SPACE>':
            simple_events.append(' ')
        elif len(e) > 1:
            pass  # get rid of <Ctrl-a> etc.
        else:
            simple_events.append(e)
    return simple_events


def is_simple_event(e):
    if isinstance(e, events.Event):
        return False
    if e in ("<Ctrl-j>", "<Ctrl-m>", "<PADENTER>", "\n", "\r", "<SPACE>"):
        return True
    if len(e) > 1:
        return False
    else:
        return True



# encoding: utf-8

# The MIT License
#
# Copyright (c) 2015-2016 Sebastian Ramacher
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import

try:
    import fcntl
    import errno
    has_fcntl = True
except ImportError:
    has_fcntl = False

try:
    import msvcrt
    has_msvcrt = True
except ImportError:
    has_msvcrt = False


class BaseLock(object):
    """Base class for file locking
    """

    def __init__(self, fileobj):
        self.fileobj = fileobj
        self.locked = False

    def acquire(self):
        pass

    def release(self):
        pass

    def __enter__(self):
        self.acquire()
        return self

    def __exit__(self, *args):
        if self.locked:
            self.release()

    def __del__(self):
        if self.locked:
            self.release()


class UnixFileLock(BaseLock):
    """Simple file locking for Unix using fcntl
    """

    def __init__(self, fileobj, mode=None):
        super(UnixFileLock, self).__init__(fileobj)

        if mode is None:
            mode = fcntl.LOCK_EX
        self.mode = mode

    def acquire(self):
        try:
            fcntl.flock(self.fileobj, self.mode)
            self.locked = True
        except IOError as e:
            if e.errno != errno.ENOLCK:
                raise e

    def release(self):
        fcntl.flock(self.fileobj, fcntl.LOCK_UN)
        self.locked = False


class WindowsFileLock(BaseLock):
    """Simple file locking for Windows using msvcrt
    """

    def __init__(self, fileobj, mode=None):
        super(WindowsFileLock, self).__init__(fileobj)

    def acquire(self):
        msvcrt.locking(self.fileobj.fileno(), msvcrt.LK_NBLCK, 1)
        self.locked = True

    def release(self):
        msvcrt.locking(self.fileobj.fileno(), msvcrt.LK_UNLCK, 1)
        self.locked = False


if has_fcntl:
    FileLock = UnixFileLock
elif has_msvcrt:
    FileLock = WindowsFileLock
else:
    FileLock = BaseLock

# vim: sw=4 ts=4 sts=4 ai et



# encoding: utf-8

# The MIT License
#
# Copyright (c) 2008 Bob Farrell
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# A simple formatter for bpython to work with Pygments.
# Pygments really kicks ass, it made it really easy to
# get the exact behaviour I wanted, thanks Pygments.:)

from __future__ import absolute_import

from pygments.formatter import Formatter
from pygments.token import Keyword, Name, Comment, String, Error, \
    Number, Operator, Token, Whitespace, Literal, Punctuation
from six import iteritems

"""These format strings are pretty ugly.
\x01 represents a colour marker, which
    can be preceded by one or two of
    the following letters:
    k, r, g, y, b, m, c, w, d
    Which represent:
    blacK, Red, Green, Yellow, Blue, Magenta,
    Cyan, White, Default
    e.g. \x01y for yellow,
        \x01gb for green on blue background

\x02 represents the bold attribute

\x03 represents the start of the actual
    text that is output (in this case it's
    a %s for substitution)

\x04 represents the end of the string; this is
    necessary because the strings are all joined
    together at the end so the parser needs them
    as delimiters

"""

Parenthesis = Token.Punctuation.Parenthesis

theme_map = {
    Keyword: 'keyword',
    Name: 'name',
    Comment: 'comment',
    String: 'string',
    Literal: 'string',
    Error: 'error',
    Number: 'number',
    Token.Literal.Number.Float: 'number',
    Operator: 'operator',
    Punctuation: 'punctuation',
    Token: 'token',
    Whitespace: 'background',
    Parenthesis: 'paren',
    Parenthesis.UnderCursor: 'operator'}


class BPythonFormatter(Formatter):
    """This is the custom formatter for bpython.
    Its format() method receives the tokensource
    and outfile params passed to it from the
    Pygments highlight() method and slops
    them into the appropriate format string
    as defined above, then writes to the outfile
    object the final formatted string.

    See the Pygments source for more info; it's pretty
    straightforward."""

    def __init__(self, color_scheme, **options):
        self.f_strings = {}
        for k, v in iteritems(theme_map):
            self.f_strings[k] = '\x01%s' % (color_scheme[v],)
            if k is Parenthesis:
                # FIXME: Find a way to make this the inverse of the current
                # background colour
                self.f_strings[k] += 'I'
        Formatter.__init__(self, **options)

    def format(self, tokensource, outfile):
        o = ''
        for token, text in tokensource:
            if text == '\n':
                continue

            while token not in self.f_strings:
                token = token.parent
            o += "%s\x03%s\x04" % (self.f_strings[token], text)
        outfile.write(o.rstrip())

# vim: sw=4 ts=4 sts=4 ai et



# encoding: utf-8

from collections import namedtuple
import inspect
import keyword
import sys

try:
    import unittest2 as unittest
except ImportError:
    import unittest

try:
    import jedi
    has_jedi = True
except ImportError:
    has_jedi = False

from bpython import autocomplete
from bpython._py3compat import py3
from bpython.test import mock

is_py34 = sys.version_info[:2] >= (3, 4)
if is_py34:
    glob_function = 'glob.iglob'
else:
    glob_function = 'glob.glob'


class TestSafeEval(unittest.TestCase):
    def test_catches_syntax_error(self):
        with self.assertRaises(autocomplete.EvaluationError):
            autocomplete.safe_eval('1re', {})


class TestFormatters(unittest.TestCase):

    def test_filename(self):
        completer = autocomplete.FilenameCompletion()
        last_part_of_filename = completer.format
        self.assertEqual(last_part_of_filename('abc'), 'abc')
        self.assertEqual(last_part_of_filename('abc/'), 'abc/')
        self.assertEqual(last_part_of_filename('abc/efg'), 'efg')
        self.assertEqual(last_part_of_filename('abc/efg/'), 'efg/')
        self.assertEqual(last_part_of_filename('/abc'), 'abc')
        self.assertEqual(last_part_of_filename('ab.c/e.f.g/'), 'e.f.g/')

    def test_attribute(self):
        self.assertEqual(autocomplete.after_last_dot('abc.edf'), 'edf')


def completer(matches):
    mock_completer = autocomplete.BaseCompletionType()
    mock_completer.matches = mock.Mock(return_value=matches)
    return mock_completer


class TestGetCompleter(unittest.TestCase):

    def test_no_completers(self):
        self.assertTupleEqual(autocomplete.get_completer([], 0, ''),
                              ([], None))

    def test_one_completer_without_matches_returns_empty_list_and_none(self):
        a = completer([])
        self.assertTupleEqual(autocomplete.get_completer([a], 0, ''),
                              ([], None))

    def test_one_completer_returns_matches_and_completer(self):
        a = completer(['a'])
        self.assertTupleEqual(autocomplete.get_completer([a], 0, ''),
                              (['a'], a))

    def test_two_completers_with_matches_returns_first_matches(self):
        a = completer(['a'])
        b = completer(['b'])
        self.assertEqual(autocomplete.get_completer([a, b], 0, ''), (['a'], a))

    def test_first_non_none_completer_matches_are_returned(self):
        a = completer([])
        b = completer(['a'])
        self.assertEqual(autocomplete.get_completer([a, b], 0, ''), ([], None))

    def test_only_completer_returns_None(self):
        a = completer(None)
        self.assertEqual(autocomplete.get_completer([a], 0, ''), ([], None))

    def test_first_completer_returns_None(self):
        a = completer(None)
        b = completer(['a'])
        self.assertEqual(autocomplete.get_completer([a, b], 0, ''), (['a'], b))


class TestCumulativeCompleter(unittest.TestCase):

    def completer(self, matches, ):
        mock_completer = autocomplete.BaseCompletionType()
        mock_completer.matches = mock.Mock(return_value=matches)
        return mock_completer

    def test_no_completers_fails(self):
        with self.assertRaises(ValueError):
            autocomplete.CumulativeCompleter([])

    def test_one_empty_completer_returns_empty(self):
        a = self.completer([])
        cumulative = autocomplete.CumulativeCompleter([a])
        self.assertEqual(cumulative.matches(3, 'abc'), set())

    def test_one_none_completer_returns_none(self):
        a = self.completer(None)
        cumulative = autocomplete.CumulativeCompleter([a])
        self.assertEqual(cumulative.matches(3, 'abc'), None)

    def test_two_completers_get_both(self):
        a = self.completer(['a'])
        b = self.completer(['b'])
        cumulative = autocomplete.CumulativeCompleter([a, b])
        self.assertEqual(cumulative.matches(3, 'abc'), set(['a', 'b']))


class TestFilenameCompletion(unittest.TestCase):

    def setUp(self):
        self.completer = autocomplete.FilenameCompletion()

    def test_locate_fails_when_not_in_string(self):
        self.assertEqual(self.completer.locate(4, "abcd"), None)

    def test_locate_succeeds_when_in_string(self):
        self.assertEqual(self.completer.locate(4, "a'bc'd"), (2, 4, 'bc'))

    def test_issue_491(self):
        self.assertNotEqual(self.completer.matches(9, '"a[a.l-1]'), None)

    @mock.patch(glob_function, new=lambda text: [])
    def test_match_returns_none_if_not_in_string(self):
        self.assertEqual(self.completer.matches(2, 'abcd'), None)

    @mock.patch(glob_function, new=lambda text: [])
    def test_match_returns_empty_list_when_no_files(self):
        self.assertEqual(self.completer.matches(2, '"a'), set())

    @mock.patch(glob_function, new=lambda text: ['abcde', 'aaaaa'])
    @mock.patch('os.path.expanduser', new=lambda text: text)
    @mock.patch('os.path.isdir', new=lambda text: False)
    @mock.patch('os.path.sep', new='/')
    def test_match_returns_files_when_files_exist(self):
        self.assertEqual(sorted(self.completer.matches(2, '"x')),
                         ['aaaaa', 'abcde'])

    @mock.patch(glob_function, new=lambda text: ['abcde', 'aaaaa'])
    @mock.patch('os.path.expanduser', new=lambda text: text)
    @mock.patch('os.path.isdir', new=lambda text: True)
    @mock.patch('os.path.sep', new='/')
    def test_match_returns_dirs_when_dirs_exist(self):
        self.assertEqual(sorted(self.completer.matches(2, '"x')),
                         ['aaaaa/', 'abcde/'])

    @mock.patch(glob_function,
                new=lambda text: ['/expand/ed/abcde', '/expand/ed/aaaaa'])
    @mock.patch('os.path.expanduser',
                new=lambda text: text.replace('~', '/expand/ed'))
    @mock.patch('os.path.isdir', new=lambda text: False)
    @mock.patch('os.path.sep', new='/')
    def test_tilde_stays_pretty(self):
        self.assertEqual(sorted(self.completer.matches(4, '"~/a')),
                         ['~/aaaaa', '~/abcde'])

    @mock.patch('os.path.sep', new='/')
    def test_formatting_takes_just_last_part(self):
        self.assertEqual(self.completer.format('/hello/there/'), 'there/')
        self.assertEqual(self.completer.format('/hello/there'), 'there')


class MockNumPy(object):
    """This is a mock numpy object that raises an error when there is an atempt
    to convert it to a boolean."""

    def __nonzero__(self):
        raise ValueError("The truth value of an array with more than one "
                         "element is ambiguous. Use a.any() or a.all()")


class TestDictKeyCompletion(unittest.TestCase):

    def test_set_of_keys_returned_when_matches_found(self):
        com = autocomplete.DictKeyCompletion()
        local = {'d': {"ab": 1, "cd": 2}}
        self.assertSetEqual(com.matches(2, "d[", locals_=local),
                            set(["'ab']", "'cd']"]))

    def test_none_returned_when_eval_error(self):
        com = autocomplete.DictKeyCompletion()
        local = {'e': {"ab": 1, "cd": 2}}
        self.assertEqual(com.matches(2, "d[", locals_=local), None)

    def test_none_returned_when_not_dict_type(self):
        com = autocomplete.DictKeyCompletion()
        local = {'l': ["ab", "cd"]}
        self.assertEqual(com.matches(2, "l[", locals_=local), None)

    def test_none_returned_when_no_matches_left(self):
        com = autocomplete.DictKeyCompletion()
        local = {'d': {"ab": 1, "cd": 2}}
        self.assertEqual(com.matches(3, "d[r", locals_=local), None)

    def test_obj_that_does_not_allow_conversion_to_bool(self):
        com = autocomplete.DictKeyCompletion()
        local = {'mNumPy': MockNumPy()}
        self.assertEqual(com.matches(7, "mNumPy[", locals_=local), None)


class Foo(object):
    a = 10

    def __init__(self):
        self.b = 20

    def method(self, x):
        pass


class OldStyleFoo:
    a = 10

    def __init__(self):
        self.b = 20

    def method(self, x):
        pass


skip_old_style = unittest.skipIf(py3,
                                 'In Python 3 there are no old style classes')


class Properties(Foo):

    @property
    def asserts_when_called(self):
        raise AssertionError("getter method called")


class Slots(object):
    __slots__ = ['a', 'b']


class TestAttrCompletion(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.com = autocomplete.AttrCompletion()

    def test_att_matches_found_on_instance(self):
        self.assertSetEqual(self.com.matches(2, 'a.', locals_={'a': Foo()}),
                            set(['a.method', 'a.a', 'a.b']))

    @skip_old_style
    def test_att_matches_found_on_old_style_instance(self):
        self.assertSetEqual(self.com.matches(2, 'a.',
                                             locals_={'a': OldStyleFoo()}),
                            set(['a.method', 'a.a', 'a.b']))
        self.assertIn(u'a.__dict__',
                      self.com.matches(4, 'a.__',
                                       locals_={'a': OldStyleFoo()}))

    @skip_old_style
    def test_att_matches_found_on_old_style_class_object(self):
        self.assertIn(u'A.__dict__',
                      self.com.matches(4, 'A.__', locals_={'A': OldStyleFoo}))

    @skip_old_style
    def test_issue536(self):
        class OldStyleWithBrokenGetAttr:
            def __getattr__(self, attr):
                raise Exception()

        locals_ = {'a': OldStyleWithBrokenGetAttr()}
        self.assertIn(u'a.__module__',
                      self.com.matches(4, 'a.__', locals_=locals_))

    def test_descriptor_attributes_not_run(self):
        com = autocomplete.AttrCompletion()
        self.assertSetEqual(com.matches(2, 'a.', locals_={'a': Properties()}),
                            set(['a.b', 'a.a', 'a.method',
                                 'a.asserts_when_called']))

    def test_slots_not_crash(self):
        com = autocomplete.AttrCompletion()
        self.assertSetEqual(com.matches(2, 'A.', locals_={'A': Slots}),
                            set(['A.b', 'A.a', 'A.mro']))


class TestExpressionAttributeCompletion(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.com = autocomplete.ExpressionAttributeCompletion()

    def test_att_matches_found_on_instance(self):
        self.assertSetEqual(self.com.matches(5, 'a[0].',
                                             locals_={'a': [Foo()]}),
                            set(['method', 'a', 'b']))

    @skip_old_style
    def test_att_matches_found_on_old_style_instance(self):
        self.assertSetEqual(self.com.matches(5, 'a[0].',
                                             locals_={'a': [OldStyleFoo()]}),
                            set(['method', 'a', 'b']))

    def test_other_getitem_methods_not_called(self):
        class FakeList(object):
            def __getitem__(inner_self, i):
                self.fail("possibly side-effecting __getitem_ method called")

        self.com.matches(5, 'a[0].', locals_={'a': FakeList()})

    def test_tuples_complete(self):
        self.assertSetEqual(self.com.matches(5, 'a[0].',
                            locals_={'a': (Foo(),)}),
                            set(['method', 'a', 'b']))

    @unittest.skip('TODO, subclasses do not complete yet')
    def test_list_subclasses_complete(self):
        class ListSubclass(list):
            pass
        self.assertSetEqual(self.com.matches(5, 'a[0].',
                            locals_={'a': ListSubclass([Foo()])}),
                            set(['method', 'a', 'b']))

    def test_getitem_not_called_in_list_subclasses_overriding_getitem(self):
        class FakeList(list):
            def __getitem__(inner_self, i):
                self.fail("possibly side-effecting __getitem_ method called")

        self.com.matches(5, 'a[0].', locals_={'a': FakeList()})

    def test_literals_complete(self):
        self.assertSetEqual(self.com.matches(10, '[a][0][0].',
                            locals_={'a': (Foo(),)}),
                            set(['method', 'a', 'b']))

    def test_dictionaries_complete(self):
        self.assertSetEqual(self.com.matches(7, 'a["b"].',
                            locals_={'a': {'b': Foo()}}),
                            set(['method', 'a', 'b']))


class TestMagicMethodCompletion(unittest.TestCase):

    def test_magic_methods_complete_after_double_underscores(self):
        com = autocomplete.MagicMethodCompletion()
        block = "class Something(object)\n    def __"
        self.assertSetEqual(com.matches(10, '    def __', current_block=block),
                            set(autocomplete.MAGIC_METHODS))


Comp = namedtuple('Completion', ['name', 'complete'])


@unittest.skipUnless(has_jedi, "jedi required")
class TestMultilineJediCompletion(unittest.TestCase):

    def test_returns_none_with_single_line(self):
        com = autocomplete.MultilineJediCompletion()
        self.assertEqual(com.matches(2, 'Va', current_block='Va', history=[]),
                         None)

    def test_returns_non_with_blank_second_line(self):
        com = autocomplete.MultilineJediCompletion()
        self.assertEqual(com.matches(0, '', current_block='class Foo():\n',
                                     history=['class Foo():']), None)

    def matches_from_completions(self, cursor, line, block, history,
                                 completions):
        with mock.patch('bpython.autocomplete.jedi.Script') as Script:
            script = Script.return_value
            script.completions.return_value = completions
            com = autocomplete.MultilineJediCompletion()
            return com.matches(cursor, line, current_block=block,
                               history=history)

    def test_completions_starting_with_different_letters(self):
        matches = self.matches_from_completions(
            2, ' a', 'class Foo:\n a', ['adsf'],
            [Comp('Abc', 'bc'), Comp('Cbc', 'bc')])
        self.assertEqual(matches, None)

    def test_completions_starting_with_different_cases(self):
        matches = self.matches_from_completions(
            2, ' a', 'class Foo:\n a', ['adsf'],
            [Comp('Abc', 'bc'), Comp('ade', 'de')])
        self.assertSetEqual(matches, set(['ade']))

    @unittest.skipUnless(is_py34, 'asyncio required')
    def test_issue_544(self):
        com = autocomplete.MultilineJediCompletion()
        code = '@asyncio.coroutine\ndef'
        history = ('import asyncio', '@asyncio.coroutin')
        com.matches(3, 'def', current_block=code, history=history)


class TestGlobalCompletion(unittest.TestCase):

    def setUp(self):
        self.com = autocomplete.GlobalCompletion()

    def test_function(self):
        def function():
            pass

        self.assertEqual(self.com.matches(8, 'function',
                                          locals_={'function': function}),
                         set(('function(', )))

    def test_completions_are_unicode(self):
        for m in self.com.matches(1, 'a', locals_={'abc': 10}):
            self.assertIsInstance(m, type(u''))

    @unittest.skipIf(py3, "in Python 3 invalid identifiers are passed through")
    def test_ignores_nonascii_encodable(self):
        self.assertEqual(self.com.matches(3, 'abc', locals_={'abcß': 10}),
                         None)

    def test_mock_kwlist(self):
        with mock.patch.object(keyword, 'kwlist', new=['abcd']):
            self.assertEqual(self.com.matches(3, 'abc', locals_={}), None)

    def test_mock_kwlist_non_ascii(self):
        with mock.patch.object(keyword, 'kwlist', new=['abcß']):
            self.assertEqual(self.com.matches(3, 'abc', locals_={}), None)


class TestParameterNameCompletion(unittest.TestCase):
    def test_set_of_params_returns_when_matches_found(self):
        def func(apple, apricot, banana, carrot):
            pass
        if py3:
            argspec = list(inspect.getfullargspec(func))
        else:
            argspec = list(inspect.getargspec(func))

        argspec = ["func", argspec, False]
        com = autocomplete.ParameterNameCompletion()
        self.assertSetEqual(com.matches(1, "a", argspec=argspec),
                            set(['apple=', 'apricot=']))
        self.assertSetEqual(com.matches(2, "ba", argspec=argspec),
                            set(['banana=']))
        self.assertSetEqual(com.matches(3, "car", argspec=argspec),
                            set(['carrot=']))



# -*- coding: utf-8 -*-

try:
    import unittest2 as unittest
except ImportError:
    import unittest

try:
    from unittest import mock
except ImportError:
    import mock

from bpython.translations import init
from bpython._py3compat import py3
from six.moves import builtins
import os


class FixLanguageTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        init(languages=['en'])


class MagicIterMock(mock.MagicMock):

    if py3:
        __next__ = mock.Mock(return_value=None)
    else:
        next = mock.Mock(return_value=None)


def builtin_target(obj):
    """Returns mock target string of a builtin"""
    return '%s.%s' % (builtins.__name__, obj.__name__)


TEST_CONFIG = os.path.join(os.path.dirname(__file__), "test.config")



# encoding: utf-8

from bpython import keys
from bpython.test import unittest


class TestCLIKeys(unittest.TestCase):
    def test_keymap_map(self):
        """Verify KeyMap.map being a dictionary with the correct
        length."""
        self.assertEqual(len(keys.cli_key_dispatch.map), 43)

    def test_keymap_setitem(self):
        """Verify keys.KeyMap correctly setting items."""
        keys.cli_key_dispatch['simon'] = 'awesome'
        self.assertEqual(keys.cli_key_dispatch['simon'], 'awesome')

    def test_keymap_delitem(self):
        """Verify keys.KeyMap correctly removing items."""
        keys.cli_key_dispatch['simon'] = 'awesome'
        del keys.cli_key_dispatch['simon']
        if 'simon' in keys.cli_key_dispatch.map:
            raise Exception('Key still exists in dictionary')

    def test_keymap_getitem(self):
        """Verify keys.KeyMap correctly looking up items."""
        self.assertEqual(keys.cli_key_dispatch['C-['], (chr(27), '^['))
        self.assertEqual(keys.cli_key_dispatch['F11'], ('KEY_F(11)',))
        self.assertEqual(keys.cli_key_dispatch['C-a'], ('\x01', '^A'))

    def test_keymap_keyerror(self):
        """Verify keys.KeyMap raising KeyError when getting undefined key"""
        with self.assertRaises(KeyError):
            keys.cli_key_dispatch['C-asdf']
            keys.cli_key_dispatch['C-qwerty']


class TestUrwidKeys(unittest.TestCase):
    def test_keymap_map(self):
        """Verify KeyMap.map being a dictionary with the correct
        length."""
        self.assertEqual(len(keys.urwid_key_dispatch.map), 64)

    def test_keymap_setitem(self):
        """Verify keys.KeyMap correctly setting items."""
        keys.urwid_key_dispatch['simon'] = 'awesome'
        self.assertEqual(keys.urwid_key_dispatch['simon'], 'awesome')

    def test_keymap_delitem(self):
        """Verify keys.KeyMap correctly removing items."""
        keys.urwid_key_dispatch['simon'] = 'awesome'
        del keys.urwid_key_dispatch['simon']
        if 'simon' in keys.urwid_key_dispatch.map:
            raise Exception('Key still exists in dictionary')

    def test_keymap_getitem(self):
        """Verify keys.KeyMap correctly looking up items."""
        self.assertEqual(keys.urwid_key_dispatch['F11'], 'f11')
        self.assertEqual(keys.urwid_key_dispatch['C-a'], 'ctrl a')
        self.assertEqual(keys.urwid_key_dispatch['M-a'], 'meta a')

    def test_keymap_keyerror(self):
        """Verify keys.KeyMap raising KeyError when getting undefined key"""
        with self.assertRaises(KeyError):
            keys.urwid_key_dispatch['C-asdf']
            keys.urwid_key_dispatch['C-qwerty']


if __name__ == '__main__':
    unittest.main()



# coding: utf8

from __future__ import unicode_literals
import itertools
import os
import pydoc
import string
import sys
from contextlib import contextmanager

from curtsies.formatstringarray import FormatStringTest, fsarray
from curtsies.fmtfuncs import cyan, bold, green, yellow, on_magenta, red

from bpython.curtsiesfrontend.events import RefreshRequestEvent
from bpython.test import mock
from bpython import config, inspection
from bpython.curtsiesfrontend.repl import BaseRepl
from bpython.curtsiesfrontend import replpainter
from bpython.curtsiesfrontend.repl import INCONSISTENT_HISTORY_MSG, \
    CONTIGUITY_BROKEN_MSG
from bpython.test import FixLanguageTestCase as TestCase, TEST_CONFIG


def setup_config():
    config_struct = config.Struct()
    config.loadini(config_struct, TEST_CONFIG)
    config_struct.cli_suggestion_width = 1
    return config_struct


class ClearEnviron(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.mock_environ = mock.patch.dict('os.environ', {}, clear=True)
        cls.mock_environ.start()
        TestCase.setUpClass()

    @classmethod
    def tearDownClass(cls):
        cls.mock_environ.stop()
        TestCase.tearDownClass()


class CurtsiesPaintingTest(FormatStringTest, ClearEnviron):
    def setUp(self):
        class TestRepl(BaseRepl):
            def _request_refresh(inner_self):
                pass
        self.repl = TestRepl(config=setup_config())
        self.repl.height, self.repl.width = (5, 10)

    @property
    def locals(self):
        return self.repl.coderunner.interp.locals

    def assert_paint(self, screen, cursor_row_col):
        array, cursor_pos = self.repl.paint()
        self.assertFSArraysEqual(array, screen)
        self.assertEqual(cursor_pos, cursor_row_col)

    def assert_paint_ignoring_formatting(self, screen, cursor_row_col=None,
                                         **paint_kwargs):
        array, cursor_pos = self.repl.paint(**paint_kwargs)
        self.assertFSArraysEqualIgnoringFormatting(array, screen)
        if cursor_row_col is not None:
            self.assertEqual(cursor_pos, cursor_row_col)


class TestCurtsiesPaintingTest(CurtsiesPaintingTest):

    def test_history_is_cleared(self):
        self.assertEqual(self.repl.rl_history.entries, [''])


class TestCurtsiesPaintingSimple(CurtsiesPaintingTest):

    def test_startup(self):
        screen = fsarray([cyan('>>> '), cyan('Welcome to')])
        self.assert_paint(screen, (0, 4))

    def test_enter_text(self):
        [self.repl.add_normal_character(c) for c in '1 + 1']
        screen = fsarray([cyan('>>> ') + bold(green('1') + cyan(' ') +
                          yellow('+') + cyan(' ') + green('1')),
                          cyan('Welcome to')])
        self.assert_paint(screen, (0, 9))

    def test_run_line(self):
        try:
            orig_stdout = sys.stdout
            sys.stdout = self.repl.stdout
            [self.repl.add_normal_character(c) for c in '1 + 1']
            self.repl.on_enter(insert_into_history=False)
            screen = fsarray(['>>> 1 + 1', '2', 'Welcome to'])
            self.assert_paint_ignoring_formatting(screen, (1, 1))
        finally:
            sys.stdout = orig_stdout

    def test_completion(self):
        self.repl.height, self.repl.width = (5, 32)
        self.repl.current_line = 'an'
        self.cursor_offset = 2
        if config.supports_box_chars():
            screen = ['>>> an',
                      '┌──────────────────────────────┐',
                      '│ and  any(                    │',
                      '└──────────────────────────────┘',
                      'Welcome to bpython! Press <F1> f']
        else:
            screen = ['>>> an',
                      '+------------------------------+',
                      '| and  any(                    |',
                      '+------------------------------+',
                      'Welcome to bpython! Press <F1> f']
        self.assert_paint_ignoring_formatting(screen, (0, 4))

    def test_argspec(self):
        def foo(x, y, z=10):
            "docstring!"
            pass
        argspec = inspection.getfuncprops('foo', foo)
        array = replpainter.formatted_argspec(argspec, 1, 30, setup_config())
        screen = [bold(cyan('foo')) + cyan(':') + cyan(' ') + cyan('(') +
                  cyan('x') + yellow(',') + yellow(' ') + bold(cyan('y')) +
                  yellow(',') + yellow(' ') + cyan('z') + yellow('=') +
                  bold(cyan('10')) + yellow(')')]
        self.assertFSArraysEqual(fsarray(array), fsarray(screen))

    def test_formatted_docstring(self):
        actual = replpainter.formatted_docstring(
            'Returns the results\n\n' 'Also has side effects',
            40, config=setup_config())
        expected = fsarray(['Returns the results', '',
                            'Also has side effects'])
        self.assertFSArraysEqualIgnoringFormatting(actual, expected)

    def test_unicode_docstrings(self):
        "A bit of a special case in Python 2"
        # issue 653

        def foo():
            u"åß∂ƒ"

        actual = replpainter.formatted_docstring(
                     foo.__doc__, 40, config=setup_config())
        expected = fsarray([u'åß∂ƒ'])
        self.assertFSArraysEqualIgnoringFormatting(actual, expected)

    def test_nonsense_docstrings(self):
        for docstring in [123, {}, [], ]:
            try:
                replpainter.formatted_docstring(
                    docstring, 40, config=setup_config())
            except Exception:
                self.fail('bad docstring caused crash: {!r}'.format(docstring))

    def test_weird_boto_docstrings(self):
        # Boto does something like this.
        # botocore: botocore/docs/docstring.py
        class WeirdDocstring(str):
            # a mighty hack. See botocore/docs/docstring.py
            def expandtabs(self, tabsize=8):
                return u'asdfåß∂ƒ'.expandtabs(tabsize)

        def foo():
            pass

        foo.__doc__ = WeirdDocstring()
        wd = pydoc.getdoc(foo)
        actual = replpainter.formatted_docstring(wd, 40, config=setup_config())
        expected = fsarray([u'asdfåß∂ƒ'])
        self.assertFSArraysEqualIgnoringFormatting(actual, expected)

    def test_paint_lasts_events(self):
        actual = replpainter.paint_last_events(4, 100, ['a', 'b', 'c'],
                                               config=setup_config())
        if config.supports_box_chars():
            expected = fsarray(["┌─┐",
                                "│c│",
                                "│b│",
                                "└─┘"])
        else:
            expected = fsarray(["+-+",
                                "|c|",
                                "|b|",
                                "+-+"])

        self.assertFSArraysEqualIgnoringFormatting(actual, expected)


@contextmanager
def output_to_repl(repl):
    old_out, old_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = repl.stdout, repl.stderr
        yield
    finally:
        sys.stdout, sys.stderr = old_out, old_err


class HigherLevelCurtsiesPaintingTest(CurtsiesPaintingTest):
    def refresh(self):
        self.refresh_requests.append(RefreshRequestEvent())

    def send_refreshes(self):
        while self.refresh_requests:
            self.repl.process_event(self.refresh_requests.pop())
            _, _ = self.repl.paint()

    def enter(self, line=None):
        """Enter a line of text, avoiding autocompletion windows

        autocomplete could still happen if the entered line has
        autocompletion that would happen then, but intermediate
        stages won't happen"""
        if line is not None:
            self.repl._set_cursor_offset(len(line), update_completion=False)
            self.repl.current_line = line
        with output_to_repl(self.repl):
            self.repl.on_enter(insert_into_history=False)
            self.assertEqual(self.repl.rl_history.entries, [''])
            self.send_refreshes()

    def undo(self):
        with output_to_repl(self.repl):
            self.repl.undo()
            self.send_refreshes()

    def setUp(self):
        self.refresh_requests = []

        class TestRepl(BaseRepl):
            def _request_refresh(inner_self):
                self.refresh()
        self.repl = TestRepl(banner='', config=setup_config())
        self.repl.height, self.repl.width = (5, 32)

    def send_key(self, key):
        self.repl.process_event('<SPACE>' if key == ' ' else key)
        self.repl.paint()  # has some side effects we need to be wary of


class TestCurtsiesRewindRedraw(HigherLevelCurtsiesPaintingTest):
    def test_rewind(self):
        self.repl.current_line = '1 + 1'
        self.enter()
        screen = ['>>> 1 + 1',
                  '2',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (2, 4))
        self.repl.undo()
        screen = ['>>> ']
        self.assert_paint_ignoring_formatting(screen, (0, 4))

    def test_rewind_contiguity_loss(self):
        self.enter('1 + 1')
        self.enter('2 + 2')
        self.enter('def foo(x):')
        self.repl.current_line = '    return x + 1'
        screen = ['>>> 1 + 1',
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> def foo(x):',
                  '...     return x + 1']
        self.assert_paint_ignoring_formatting(screen, (5, 8))
        self.repl.scroll_offset = 1
        self.assert_paint_ignoring_formatting(screen[1:], (4, 8))
        self.undo()
        screen = ['2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (3, 4))
        self.undo()
        screen = ['2',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (1, 4))
        self.undo()
        screen = [CONTIGUITY_BROKEN_MSG[:self.repl.width],
                  '>>> ',
                  '',
                  '',
                  '',
                  ' ']  # TODO why is that there? Necessary?
        self.assert_paint_ignoring_formatting(screen, (1, 4))
        screen = ['>>> ']
        self.assert_paint_ignoring_formatting(screen, (0, 4))

    def test_inconsistent_history_doesnt_happen_if_onscreen(self):
        self.enter('1 + 1')
        screen = ['>>> 1 + 1',
                  '2',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (2, 4))
        self.enter("2 + 2")
        screen = ['>>> 1 + 1',
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (4, 4))
        self.repl.display_lines[0] = self.repl.display_lines[0] * 2
        self.undo()
        screen = ['>>> 1 + 1',
                  '2',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (2, 4))

    def test_rewind_inconsistent_history(self):
        self.enter('1 + 1')
        self.enter('2 + 2')
        self.enter('3 + 3')
        screen = ['>>> 1 + 1',
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> 3 + 3',
                  '6',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (6, 4))
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[2:], (4, 4))
        self.repl.display_lines[0] = self.repl.display_lines[0] * 2
        self.undo()
        screen = [INCONSISTENT_HISTORY_MSG[:self.repl.width],
                  '>>> 2 + 2',
                  '4',
                  '>>> ',
                  '',
                  ' ']
        self.assert_paint_ignoring_formatting(screen, (3, 4))
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[1:-2], (2, 4))
        self.assert_paint_ignoring_formatting(screen[1:-2], (2, 4))

    def test_rewind_inconsistent_history_more_lines_same_screen(self):
        self.repl.width = 60
        sys.a = 5
        self.enter('import sys')
        self.enter('for i in range(sys.a):')
        self.enter('    print(sys.a)')
        self.enter('')
        self.enter('1 + 1')
        self.enter('2 + 2')
        screen = ['>>> import sys',
                  '>>> for i in range(sys.a):',
                  '...     print(sys.a)',
                  '... ',
                  '5',
                  '5',
                  '5',
                  '5',
                  '5',
                  '>>> 1 + 1',
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (13, 4))
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[9:], (4, 4))
        sys.a = 6
        self.undo()
        screen = [INCONSISTENT_HISTORY_MSG[:self.repl.width],
                  '6',
                  # everything will jump down a line - that's perfectly
                  # reasonable
                  '>>> 1 + 1',
                  '2',
                  '>>> ',
                  ' ']
        self.assert_paint_ignoring_formatting(screen, (4, 4))
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[1:-1], (3, 4))

    def test_rewind_inconsistent_history_more_lines_lower_screen(self):
        self.repl.width = 60
        sys.a = 5
        self.enter("import sys")
        self.enter("for i in range(sys.a):")
        self.enter("    print(sys.a)")
        self.enter("")
        self.enter("1 + 1")
        self.enter("2 + 2")
        screen = [">>> import sys",
                  ">>> for i in range(sys.a):",
                  "...     print(sys.a)",
                  '... ',
                  '5',
                  '5',
                  '5',
                  '5',
                  '5',
                  '>>> 1 + 1',
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (13, 4))
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[9:], (4, 4))
        sys.a = 8
        self.undo()
        screen = [INCONSISTENT_HISTORY_MSG[:self.repl.width],
                  '8',
                  '8',
                  '8',
                  '>>> 1 + 1',
                  '2',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen)
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[-5:])

    def test_rewind_inconsistent_history_more_lines_raise_screen(self):
        self.repl.width = 60
        sys.a = 5
        self.enter("import sys")
        self.enter("for i in range(sys.a):")
        self.enter("    print(sys.a)")
        self.enter("")
        self.enter("1 + 1")
        self.enter("2 + 2")
        screen = [">>> import sys",
                  ">>> for i in range(sys.a):",
                  "...     print(sys.a)",
                  '... ',
                  '5',
                  '5',
                  '5',
                  '5',
                  '5',
                  '>>> 1 + 1',
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (13, 4))
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[9:], (4, 4))
        sys.a = 1
        self.undo()
        screen = [INCONSISTENT_HISTORY_MSG[:self.repl.width],
                  '1',
                  '>>> 1 + 1',
                  '2',
                  '>>> ',
                  ' ']
        self.assert_paint_ignoring_formatting(screen)
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[1:-1])

    def test_rewind_history_not_quite_inconsistent(self):
        self.repl.width = 50
        sys.a = 5
        self.enter("for i in range(__import__('sys').a):")
        self.enter("    print(i)")
        self.enter("")
        self.enter("1 + 1")
        self.enter("2 + 2")
        screen = [">>> for i in range(__import__('sys').a):",
                  "...     print(i)",
                  "... ",
                  '0',
                  '1',
                  '2',
                  '3',
                  '4',
                  '>>> 1 + 1',
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (12, 4))
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[8:], (4, 4))
        sys.a = 6
        self.undo()
        screen = ['5',
                  # everything will jump down a line - that's perfectly
                  # reasonable
                  '>>> 1 + 1',
                  '2',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (3, 4))

    def test_rewind_barely_consistent(self):
        self.enter("1 + 1")
        self.enter("2 + 2")
        self.enter("3 + 3")
        screen = [">>> 1 + 1",
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> 3 + 3',
                  '6',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (6, 4))
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[2:], (4, 4))
        self.repl.display_lines[2] = self.repl.display_lines[2] * 2
        self.undo()
        screen = ['>>> 2 + 2',
                  '4',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (2, 4))

    def test_clear_screen(self):
        self.enter("1 + 1")
        self.enter("2 + 2")
        screen = [">>> 1 + 1",
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (4, 4))
        self.repl.request_paint_to_clear_screen = True
        screen = [">>> 1 + 1",
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ', '', '', '', '']
        self.assert_paint_ignoring_formatting(screen, (4, 4))

    def test_scroll_down_while_banner_visible(self):
        self.repl.status_bar.message('STATUS_BAR')
        self.enter("1 + 1")
        self.enter("2 + 2")
        screen = [">>> 1 + 1",
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ',
                  'STATUS_BAR                      ']
        self.assert_paint_ignoring_formatting(screen, (4, 4))
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[1:], (3, 4))

    def test_clear_screen_while_banner_visible(self):
        self.repl.status_bar.message('STATUS_BAR')
        self.enter("1 + 1")
        self.enter("2 + 2")
        screen = [">>> 1 + 1",
                  '2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ',
                  'STATUS_BAR                      ']
        self.assert_paint_ignoring_formatting(screen, (4, 4))
        self.repl.scroll_offset += len(screen) - self.repl.height
        self.assert_paint_ignoring_formatting(screen[1:], (3, 4))

        self.repl.request_paint_to_clear_screen = True
        screen = ['2',
                  '>>> 2 + 2',
                  '4',
                  '>>> ',
                  '', '', '',
                  'STATUS_BAR                      ']
        self.assert_paint_ignoring_formatting(screen, (3, 4))

    def test_cursor_stays_at_bottom_of_screen(self):
        """infobox showing up during intermediate render was causing this to
        fail, #371"""
        self.repl.width = 50
        self.repl.current_line = "__import__('random').__name__"
        with output_to_repl(self.repl):
            self.repl.on_enter(insert_into_history=False)
        screen = [">>> __import__('random').__name__",
                  "'random'"]
        self.assert_paint_ignoring_formatting(screen)

        with output_to_repl(self.repl):
            self.repl.process_event(self.refresh_requests.pop())
        screen = [">>> __import__('random').__name__",
                  "'random'",
                  ""]
        self.assert_paint_ignoring_formatting(screen)

        with output_to_repl(self.repl):
            self.repl.process_event(self.refresh_requests.pop())
        screen = [">>> __import__('random').__name__",
                  "'random'",
                  ">>> "]
        self.assert_paint_ignoring_formatting(screen, (2, 4))

    def test_unhighlight_paren_bugs(self):
        """two previous bugs, parent didn't highlight until next render
        and paren didn't unhighlight until enter"""
        self.assertEqual(self.repl.rl_history.entries, [''])
        self.enter('(')
        self.assertEqual(self.repl.rl_history.entries, [''])
        screen = [">>> (",
                  "... "]
        self.assertEqual(self.repl.rl_history.entries, [''])
        self.assert_paint_ignoring_formatting(screen)
        self.assertEqual(self.repl.rl_history.entries, [''])

        with output_to_repl(self.repl):
            self.assertEqual(self.repl.rl_history.entries, [''])
            self.repl.process_event(')')
            self.assertEqual(self.repl.rl_history.entries, [''])
        screen = fsarray([cyan(">>> ") + on_magenta(bold(red('('))),
                         green("... ") + on_magenta(bold(red(')')))])
        self.assert_paint(screen, (1, 5))

        with output_to_repl(self.repl):
            self.repl.process_event(' ')
        screen = fsarray([cyan(">>> ") + yellow('('),
                         green("... ") + yellow(')') + bold(cyan(" "))])
        self.assert_paint(screen, (1, 6))

    def test_472(self):
        [self.send_key(c) for c in "(1, 2, 3)"]
        with output_to_repl(self.repl):
            self.send_key('\n')
            self.send_refreshes()
            self.send_key('<UP>')
            self.repl.paint()
            [self.send_key('<LEFT>') for _ in range(4)]
            self.send_key('<BACKSPACE>')
            self.send_key('4')
            self.repl.on_enter()
            self.send_refreshes()
        screen = [">>> (1, 2, 3)",
                  '(1, 2, 3)',
                  '>>> (1, 4, 3)',
                  '(1, 4, 3)',
                  '>>> ']
        self.assert_paint_ignoring_formatting(screen, (4, 4))


def completion_target(num_names, chars_in_first_name=1):
    class Class(object):
        pass

    if chars_in_first_name < 1:
        raise ValueError('need at least one char in each name')
    elif chars_in_first_name == 1 and num_names > len(string.ascii_letters):
        raise ValueError('need more chars to make so many names')

    names = gen_names()
    if num_names > 0:
        setattr(Class, 'a' * chars_in_first_name, 1)
        next(names)  # use the above instead of first name
    for _, name in zip(range(num_names - 1), names):
        setattr(Class, name, 0)

    return Class()


def gen_names():
    for letters in itertools.chain(
            itertools.combinations_with_replacement(string.ascii_letters, 1),
            itertools.combinations_with_replacement(string.ascii_letters, 2)):
        yield ''.join(letters)


class TestCompletionHelpers(TestCase):
    def test_gen_names(self):
        self.assertEqual(list(zip([1, 2, 3], gen_names())),
                         [(1, 'a'), (2, 'b'), (3, 'c')])

    def test_completion_target(self):
        target = completion_target(14)
        self.assertEqual(len([x for x in dir(target)
                              if not x.startswith('_')]),
                         14)


class TestCurtsiesInfoboxPaint(HigherLevelCurtsiesPaintingTest):
    def test_simple(self):
        self.repl.width, self.repl.height = (20, 30)
        self.locals['abc'] = completion_target(3, 50)
        self.repl.current_line = 'abc'
        self.repl.cursor_offset = 3
        self.repl.process_event('.')
        screen = ['>>> abc.',
                  '+------------------+',
                  '| aaaaaaaaaaaaaaaa |',
                  '| b                |',
                  '| c                |',
                  '+------------------+']
        self.assert_paint_ignoring_formatting(screen, (0, 8))

    def test_fill_screen(self):
        self.repl.width, self.repl.height = (20, 15)
        self.locals['abc'] = completion_target(20, 100)
        self.repl.current_line = 'abc'
        self.repl.cursor_offset = 3
        self.repl.process_event('.')
        screen = ['>>> abc.',
                  '+------------------+',
                  '| aaaaaaaaaaaaaaaa |',
                  '| b                |',
                  '| c                |',
                  '| d                |',
                  '| e                |',
                  '| f                |',
                  '| g                |',
                  '| h                |',
                  '| i                |',
                  '| j                |',
                  '| k                |',
                  '| l                |',
                  '+------------------+']
        self.assert_paint_ignoring_formatting(screen, (0, 8))

    def test_lower_on_screen(self):
        self.repl.get_top_usable_line = lambda: 10  # halfway down terminal
        self.repl.width, self.repl.height = (20, 15)
        self.locals['abc'] = completion_target(20, 100)
        self.repl.current_line = 'abc'
        self.repl.cursor_offset = 3
        self.repl.process_event('.')
        screen = ['>>> abc.',
                  '+------------------+',
                  '| aaaaaaaaaaaaaaaa |',
                  '| b                |',
                  '| c                |',
                  '| d                |',
                  '| e                |',
                  '| f                |',
                  '| g                |',
                  '| h                |',
                  '| i                |',
                  '| j                |',
                  '| k                |',
                  '| l                |',
                  '+------------------+']
        # behavior before issue #466
        self.assert_paint_ignoring_formatting(
                screen, try_preserve_history_height=0)
        self.assert_paint_ignoring_formatting(
                screen, min_infobox_height=100)
        # behavior after issue #466
        screen = ['>>> abc.',
                  '+------------------+',
                  '| aaaaaaaaaaaaaaaa |',
                  '| b                |',
                  '| c                |',
                  '+------------------+']
        self.assert_paint_ignoring_formatting(screen)

    def test_at_bottom_of_screen(self):
        self.repl.get_top_usable_line = lambda: 17  # two lines from bottom
        self.repl.width, self.repl.height = (20, 15)
        self.locals['abc'] = completion_target(20, 100)
        self.repl.current_line = 'abc'
        self.repl.cursor_offset = 3
        self.repl.process_event('.')
        screen = ['>>> abc.',
                  '+------------------+',
                  '| aaaaaaaaaaaaaaaa |',
                  '| b                |',
                  '| c                |',
                  '| d                |',
                  '| e                |',
                  '| f                |',
                  '| g                |',
                  '| h                |',
                  '| i                |',
                  '| j                |',
                  '| k                |',
                  '| l                |',
                  '+------------------+']
        # behavior before issue #466
        self.assert_paint_ignoring_formatting(
                screen, try_preserve_history_height=0)
        self.assert_paint_ignoring_formatting(
                screen, min_infobox_height=100)
        # behavior after issue #466
        screen = ['>>> abc.',
                  '+------------------+',
                  '| aaaaaaaaaaaaaaaa |',
                  '| b                |',
                  '| c                |',
                  '+------------------+']
        self.assert_paint_ignoring_formatting(screen)



# encoding: utf-8

import sys

from six.moves import builtins


def resetquit(builtins):
    """Redefine builtins 'quit' and 'exit' not so close stdin

    """
    def __call__(self, code=None):
        raise SystemExit(code)
    __call__.__name__ = 'FakeQuitCall'
    builtins.quit.__class__.__call__ = __call__


def monkeypatch_quit():
    if 'site' in sys.modules:
        resetquit(builtins)



# -*- coding: utf-8 -*-

import os

from bpython._py3compat import py3
from bpython import inspection
from bpython.test import unittest
from bpython.test.fodder import encoding_ascii
from bpython.test.fodder import encoding_latin1
from bpython.test.fodder import encoding_utf8


foo_ascii_only = u'''def foo():
    """Test"""
    pass
'''

foo_non_ascii = u'''def foo():
    """Test äöü"""
    pass
'''


class OldCallable:
    def __call__(self):
        pass


class Callable(object):
    def __call__(self):
        pass


class OldNoncallable:
    pass


class Noncallable(object):
    pass


def spam():
    pass


class CallableMethod(object):
    def method(self):
        pass


class TestInspection(unittest.TestCase):
    def test_is_callable(self):
        self.assertTrue(inspection.is_callable(spam))
        self.assertTrue(inspection.is_callable(Callable))
        self.assertTrue(inspection.is_callable(Callable()))
        self.assertTrue(inspection.is_callable(OldCallable))
        self.assertTrue(inspection.is_callable(OldCallable()))
        self.assertFalse(inspection.is_callable(Noncallable()))
        self.assertFalse(inspection.is_callable(OldNoncallable()))
        self.assertFalse(inspection.is_callable(None))
        self.assertTrue(inspection.is_callable(CallableMethod().method))

    @unittest.skipIf(py3, 'old-style classes only exist in Python 2')
    def test_is_new_style_py2(self):
        self.assertTrue(inspection.is_new_style(spam))
        self.assertTrue(inspection.is_new_style(Noncallable))
        self.assertFalse(inspection.is_new_style(OldNoncallable))
        self.assertTrue(inspection.is_new_style(Noncallable()))
        self.assertFalse(inspection.is_new_style(OldNoncallable()))
        self.assertTrue(inspection.is_new_style(None))

    @unittest.skipUnless(py3, 'only in Python 3 are all classes new-style')
    def test_is_new_style_py3(self):
        self.assertTrue(inspection.is_new_style(spam))
        self.assertTrue(inspection.is_new_style(Noncallable))
        self.assertTrue(inspection.is_new_style(OldNoncallable))
        self.assertTrue(inspection.is_new_style(Noncallable()))
        self.assertTrue(inspection.is_new_style(OldNoncallable()))
        self.assertTrue(inspection.is_new_style(None))


    def test_parsekeywordpairs(self):
        # See issue #109
        def fails(spam=['-a', '-b']):
            pass

        default_arg_repr = "['-a', '-b']"
        self.assertEqual(str(['-a', '-b']), default_arg_repr,
                         'This test is broken (repr does not match), fix me.')

        argspec = inspection.getfuncprops('fails', fails)
        defaults = argspec.argspec.defaults
        self.assertEqual(str(defaults[0]), default_arg_repr)

    def test_pasekeywordpairs_string(self):
        def spam(eggs="foo, bar"):
            pass

        defaults = inspection.getfuncprops("spam", spam).argspec.defaults
        self.assertEqual(repr(defaults[0]), "'foo, bar'")

    def test_parsekeywordpairs_multiple_keywords(self):
        def spam(eggs=23, foobar="yay"):
            pass

        defaults = inspection.getfuncprops("spam", spam).argspec.defaults
        self.assertEqual(repr(defaults[0]), "23")
        self.assertEqual(repr(defaults[1]), "'yay'")

    def test_get_encoding_ascii(self):
        self.assertEqual(inspection.get_encoding(encoding_ascii), 'ascii')
        self.assertEqual(inspection.get_encoding(encoding_ascii.foo), 'ascii')

    def test_get_encoding_latin1(self):
        self.assertEqual(inspection.get_encoding(encoding_latin1), 'latin1')
        self.assertEqual(inspection.get_encoding(encoding_latin1.foo),
                         'latin1')

    def test_get_encoding_utf8(self):
        self.assertEqual(inspection.get_encoding(encoding_utf8), 'utf-8')
        self.assertEqual(inspection.get_encoding(encoding_utf8.foo), 'utf-8')

    def test_get_source_ascii(self):
        self.assertEqual(inspection.get_source_unicode(encoding_ascii.foo),
                         foo_ascii_only)

    def test_get_source_utf8(self):
        self.assertEqual(inspection.get_source_unicode(encoding_utf8.foo),
                         foo_non_ascii)

    def test_get_source_latin1(self):
        self.assertEqual(inspection.get_source_unicode(encoding_latin1.foo),
                         foo_non_ascii)

    def test_get_source_file(self):
        path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                            'fodder')

        encoding = inspection.get_encoding_file(
            os.path.join(path, 'encoding_ascii.py'))
        self.assertEqual(encoding, 'ascii')
        encoding = inspection.get_encoding_file(
            os.path.join(path, 'encoding_latin1.py'))
        self.assertEqual(encoding, 'latin1')
        encoding = inspection.get_encoding_file(
            os.path.join(path, 'encoding_utf8.py'))
        self.assertEqual(encoding, 'utf-8')


if __name__ == '__main__':
    unittest.main()



# encoding: utf-8

from __future__ import unicode_literals

from bpython import importcompletion
from bpython.test import unittest


class TestSimpleComplete(unittest.TestCase):

    def setUp(self):
        self.original_modules = importcompletion.modules
        importcompletion.modules = ['zzabc', 'zzabd', 'zzefg', 'zzabc.e',
                                    'zzabc.f']

    def tearDown(self):
        importcompletion.modules = self.original_modules

    def test_simple_completion(self):
        self.assertSetEqual(importcompletion.complete(10, 'import zza'),
                            set(['zzabc', 'zzabd']))

    def test_package_completion(self):
        self.assertSetEqual(importcompletion.complete(13, 'import zzabc.'),
                            set(['zzabc.e', 'zzabc.f']))


class TestRealComplete(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        for _ in importcompletion.find_iterator:
            pass
        __import__('sys')
        __import__('os')

    @classmethod
    def tearDownClass(cls):
        importcompletion.find_iterator = importcompletion.find_all_modules()
        importcompletion.modules = set()

    def test_from_attribute(self):
        self.assertSetEqual(
            importcompletion.complete(19, 'from sys import arg'),
            set(['argv']))

    def test_from_attr_module(self):
        self.assertSetEqual(importcompletion.complete(9, 'from os.p'),
                            set(['os.path']))

    def test_from_package(self):
        self.assertSetEqual(importcompletion.complete(17, 'from xml import d'),
                            set(['dom']))



# encoding: utf-8

"""Tools for preparing code to be run in the REPL (removing blank lines,
etc)"""

from bpython.lazyre import LazyReCompile

# TODO specifically catch IndentationErrors instead of any syntax errors


indent_empty_lines_re = LazyReCompile(r'\s*')
tabs_to_spaces_re = LazyReCompile(r'^\t+')


def indent_empty_lines(s, compiler):
    """Indents blank lines that would otherwise cause early compilation

    Only really works if starting on a new line"""
    lines = s.split('\n')
    ends_with_newline = False
    if lines and not lines[-1]:
        ends_with_newline = True
        lines.pop()
    result_lines = []

    for p_line, line, n_line in zip([''] + lines[:-1], lines,
                                    lines[1:] + ['']):
        if len(line) == 0:
            p_indent = indent_empty_lines_re.match(p_line).group()
            n_indent = indent_empty_lines_re.match(n_line).group()
            result_lines.append(min([p_indent, n_indent], key=len) + line)
        else:
            result_lines.append(line)

    return '\n'.join(result_lines) + ('\n' if ends_with_newline else '')


def leading_tabs_to_spaces(s):
    lines = s.split('\n')
    result_lines = []

    def tab_to_space(m):
        return len(m.group()) * 4 * ' '
    for line in lines:
        result_lines.append(tabs_to_spaces_re.sub(tab_to_space, line))
    return '\n'.join(result_lines)


def preprocess(s, compiler):
    return indent_empty_lines(leading_tabs_to_spaces(s), compiler)
